Package: BEclear
Type: Package
Title: Correction of batch effects in DNA methylation data
Version: 2.26.0
Date: 2023-04-06
Authors@R: c(person("Livia", "Rasp", email = "livia.rasp@gmail.com", role = c("aut", "cre"), comment = c(ORCID = "0000-0003-0164-2163")),
             person("Markus", "Merl", role = "aut"),
             person("Ruslan", "Akulenko", role = "aut"))
Description: Provides functions to detect and correct for batch effects in
             DNA methylation data. The core function  is based on latent factor
             models and can also be used to predict missing values in any other 
             matrix containing real numbers.
License: GPL-3
SystemRequirements: C++11
Depends: BiocParallel (>= 1.14.2)
Imports: futile.logger, Rdpack, Matrix, data.table (>= 1.11.8), Rcpp,
        abind, stats, graphics, utils, methods, dixonTest, ids
Suggests: testthat, BiocStyle, knitr, rmarkdown, pander, seewave
VignetteBuilder: knitr
biocViews: BatchEffect, DNAMethylation, Software, Preprocessing,
        StatisticalMethod
RoxygenNote: 7.2.2
RdMacros: Rdpack
Encoding: UTF-8
URL: https://github.com/uds-helms/BEclear
BugReports: https://github.com/uds-helms/BEclear/issues
LinkingTo: Rcpp
git_url: https://git.bioconductor.org/packages/BEclear
git_branch: RELEASE_3_22
git_last_commit: 575a6df
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: yes
Packaged: 2025-10-30 02:43:04 UTC; biocbuild
Author: Livia Rasp [aut, cre] (ORCID: <https://orcid.org/0000-0003-0164-2163>),
  Markus Merl [aut],
  Ruslan Akulenko [aut]
Maintainer: Livia Rasp <livia.rasp@gmail.com>

# Generated by roxygen2: do not edit by hand

export(calcBatchEffects)
export(calcScore)
export(calcSummary)
export(clearBEgenes)
export(correctBatchEffect)
export(countValuesToPredict)
export(findOutsideValues)
export(imputeMissingData)
export(localLoss)
export(loss)
export(makeBoxplot)
export(preprocessBEclear)
export(replaceOutsideValues)
import(BiocParallel)
import(Matrix)
import(Rcpp)
import(Rdpack)
import(data.table)
import(futile.logger)
importFrom(Matrix,norm)
importFrom(Rdpack,reprompt)
importFrom(abind,abind)
importFrom(dixonTest,dixonTest)
importFrom(futile.logger,flog.info)
importFrom(graphics,boxplot)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(ids,random_id)
importFrom(methods,as)
importFrom(methods,is)
importFrom(stats,ks.test)
importFrom(stats,median)
importFrom(stats,p.adjust)
importFrom(stats,rnorm)
importFrom(utils,write.table)
useDynLib(BEclear)

[![Travis-CI Build Status](https://travis-ci.org/uds-helms/BEclear.svg?branch=master)](https://travis-ci.org/uds-helms/BEclear)
[![Coverage Status](https://img.shields.io/codecov/c/github/uds-helms/BEclear/master.svg)](https://codecov.io/github/uds-helms/BEclear?branch=master)


# BEclear

## Description

This package provides functions to detect and correct for batch effects in
DNA methylation data. The core function for the data imputation is based on 
latent factor models and can also be used to predict missing values in any other 
matrix containing real numbers.

## Installation

```r
# Installation from Bioconductor
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("BEclear")
```

```r
# Installation of the development version from GitHub
if (!requireNamespace("devtools", quietly = TRUE))
    install.packages("devtools")
devtools::install_github("uds-helms/BEclear", build_opts = c())
```

## Usage example

For example code you can either read the [vignette](https://bioconductor.org/packages/devel/bioc/vignettes/BEclear/inst/doc/BEclear.html) on Bioconductor. Or you can open it local, after you installed the BEclear package with following command:

```r
browseVignettes("BEclear")
```

## Planned features

- [Treating Data-Sets Without Batches](https://github.com/uds-helms/BEclear/issues/22)
- [Make Saving Temporary Data Optional](https://github.com/uds-helms/BEclear/issues/21)
- [Bias Modelling](https://github.com/uds-helms/BEclear/issues/18)
- [ALS](https://github.com/uds-helms/BEclear/issues/19)
- [Test For Convergence Durin Epochs](https://github.com/uds-helms/BEclear/issues/23)
- [Testing BEclear on other data-sets](https://github.com/uds-helms/BEclear/issues/24)
- [After Merging Blocks Continue GD](https://github.com/uds-helms/BEclear/issues/25)

If you have a suggestion
you could either open an [issue](https://github.com/uds-helms/BEclear/issues) or 
write an email under the following address: [Livia.Rasp at gmail.com](mailto:Livia.Rasp@gmail.com)


## Citation

Akulenko, R., Merl, M., & Helms, V. (2016). BEclear: Batch effect detection and 
adjustment in DNA methylation data. PLoS ONE, 11(8), 1–17.
https://doi.org/10.1371/journal.pone.0159921

#' combineBlocks
#'
#' @keywords internal
#'
#' @description  load and combine blocks
#'
#' @return the combined matrix consisting of the individual blocks
combineBlocks <- function(blockFrame, rowPos, colPos, dir) {
  D1 <- NULL
  blockCounter <- 1
  for (i in seq_len(rowPos[1])) {
    rowStart <- blockFrame[[2]][[blockCounter]]
    rowStop <- blockFrame[[3]][[blockCounter]]
    for (j in seq_len(colPos[1])) {
      colStart <- blockFrame[[4]][[blockCounter]]
      colStop <- blockFrame[[5]][[blockCounter]]
      ## special case, only one column block
      if (colPos[1] == 1) {
        blockName <- paste("D", blockCounter, sep = "")
        filename <- paste(blockName, "RData", sep = ".")
        file <- paste(dir, filename, sep = "/")
        load(file)
        assign(blockName, D1)
        frame <- D1
        currentRowGroup <- as.data.frame(frame)
        blockCounter <- blockCounter + 1
      } else {
        if (j == 1) {
          blockName <- paste("D", blockCounter, sep = "")
          filename <- paste(blockName, "RData", sep = ".")
          file <- paste(dir, filename, sep = "/")
          load(file)
          assign(blockName, D1)
          frame <- D1
          currentRowGroup <- as.data.frame(frame)
          blockCounter <- blockCounter + 1
        }
        else {
          blockName <- paste("D", blockCounter, sep = "")
          filename <- paste(blockName, "RData", sep = ".")
          file <- paste(dir, filename, sep = "/")
          load(file)
          assign(blockName, D1)
          frame <- D1
          currentRowGroup <- cbind(
            currentRowGroup,
            as.data.frame(frame)
          )
          blockCounter <- blockCounter + 1
        }
      }
    }
    ## special case, only one row block
    if (rowPos[1] == 1) {
      predictedGenes <- currentRowGroup
    } else {
      if (i == 1) {
        predictedGenes <- currentRowGroup
      }
      else {
        predictedGenes <- rbind(predictedGenes, currentRowGroup)
      }
    }
  }
  return(as.matrix(predictedGenes))
}

#' loss
#' @param L a matrix describing the effects of the features
#' @param R a matrix describing the effects of the samples
#' @param D a matrix containing the measured values
#' @param lambda constant that controls the extent of regularization during the
#' gradient descent
#'
#' @description  computation of the loss of factorization LR
#'
#' @importFrom Matrix norm
#' @export loss
#'
#' @return a list containing the loss calculated and the error matrix
loss <- function(L, R, lambda, D) {
  error_matrix <- D - (L %*% R)

  squared_error <- sum((error_matrix)^2, na.rm = TRUE)

  cost <- lambda * ((norm(L, type = "F")^2) + (norm(R, type = "F")^2))

  return(list(loss = squared_error + cost, error_matrix = error_matrix))
}

testthat::context("calculating the loss for matrices L and R")

testthat::test_that("matrices with integer results", {
  res1 <- loss(
    L = matrix(c(1, 2)), R = t(matrix(c(3, 4))),
    D = matrix(c(1, 1, 1, 1), nrow = 2),
    lambda = 0.2
  )
  testthat::expect_equal(res1$loss, 93)
  testthat::expect_equal(res1$error_matrix, matrix(c(-2, -5, -3, -7), ncol = 2))
})

#' clearBEgenes
#'
#' @aliases clearBEgenes
#'
#' @seealso \code{\link{calcBatchEffects}}
#' @seealso \code{\link{calcSummary}}
#' @seealso \code{\link{correctBatchEffect}}
#'
#' @title Prepare a data matrix for the BEclear function
#'
#' @description A function that simply sets all values to NA which were
#' previously found by median value comparison and p-value calculation and are
#' stored in a summary. The summary defines which values in the data matrix are
#' set to NA.
#'
#' @details All entries belonging to genes stated in the summary are set to NA
#' for the corresponding batches in the data matrix. Please look at the
#' descriptions of \code{\link{calcBatchEffects}}  for
#' more detailed information about the data which should be contained in the
#' summary data.frame.
#'
#' @param data any matrix filled with beta values, column names have to be
#' sample_ids corresponding to the ids listed in "samples", row names have to be
#' gene names.
#' @param samples data frame with two columns, the first column has to contain
#' the sample numbers, the second column has to contain the corresponding batch
#' number. Colnames have to be named as "sample_id" and "batch_id".
#' @param summary a summary data.frame containing the columns "gene", "batch",
#' "median" and "p-value" and consists of all genes which were found in the
#' median and p-value calculations, see \code{\link{calcSummary}} function for
#' more details.
#'
#' @export clearBEgenes
#' @import futile.logger
#' @usage clearBEgenes(data, samples, summary)
#'
#' @return A data matrix with the same dimensions as well as the same column
#' and row names as the input data matrix is returned, all entries which are
#' defined in the summary are now set to NA.
#'
#' @examples
#' ## Shortly running example. For a more realistic example that takes
#' ## some more time, run the same procedure with the full BEclearData
#' ## dataset.
#' 
#' ## Whole procedure that has to be done to use this function.
#' data(BEclearData)
#' ex.data <- ex.data[31:90, 7:26]
#' ex.samples <- ex.samples[7:26, ]
#' 
#' ## Calculate the batch effects
#' batchEffects <- calcBatchEffects(data = ex.data, samples = ex.samples,
#' adjusted = TRUE, method = "fdr")
#' meds <- batchEffects$med
#' pvals <- batchEffects$pval
#' 
#' ## Summarize p-values and median differences for batch affected genes
#' sum <- calcSummary(medians = meds, pvalues = pvals)
#' 
#' ## Set values for summarized BEgenes to NA
#' clearedMatrix <- clearBEgenes(data = ex.data, samples = ex.samples, summary = sum)
clearBEgenes <- function(data, samples, summary) {
  ## set beta values in data to NA for all found genes
  for (i in seq_len(nrow(summary)))
  {
    data[summary$gene[i], as.character(
      samples$sample_id[samples$batch_id == summary$batch_id[i]]
    )] <- NA
  }
  amount <- sum(is.na(data)) / (nrow(data) * ncol(data))
  flog.info("Removing values with batch effect:")
  flog.info(paste(sum(is.na(data)), "values (", amount * 100,
    "% of the data) set to NA",
    sep = " "
  ))
  return(data)
}

#' replaceOutsideValues
#'
#' @aliases replaceOutsideValues
#'
#' @title Replace DNA methylation values out of the boundaries
#'
#' @seealso \code{\link{findOutsideValues}}
#' @seealso \code{\link{correctBatchEffect}}
#'
#' @description A method which replaces values below 0 or beyond 1 contained in
#' the input matrix. These wrong entries are replaced by 0 or 1, respectively.
#' Note that this method is especially designed for DNA methylation data.
#'
#' @details Note that this method is especially designed to run after the batch
#' effect correction of DNA methylation data, e.g. with the
#' \code{\link{imputeMissingData}} method. It can happen, that the predicted
#' values are lying slightly below the lower bound of 0 or beyond the upper
#' bound of 1. This method finds these inaccurately predicted entries. Another
#' method called \code{\link{replaceOutsideValues}} replaces these values either
#' by 0 or 1, respectively. Another method called \code{\link{findOutsideValues}}
#' returns a list of existing wrong values and can be run before the
#' replacement.
#'
#' @param data any matrix filled with values that normally should be bounded
#' between 0 and 1.
#'
#' @export replaceOutsideValues
#' @import futile.logger
#' @usage replaceOutsideValues(data)
#'
#' @return Returns the input matrix with every value previously below 0 changed
#' to 0 and every value previously beyond 1 changed to 1.
#'
#' @examples
#' data(BEclearCorrected)
#' # Replace wrongly predicted values
#' corrected <- replaceOutsideValues(data = ex.corrected.data)
replaceOutsideValues <- function(data) {
  flog.info("Replacing values below 0 or above 1:")
  counter <- sum(data > 1) + sum(data < 0)

  data[data > 1] <- 1
  data[data < 0] <- 0
  flog.info(paste(counter, "values replaced"))

  return(data)
}

#' imputeMissingDataForBlock
#'
#' @import futile.logger
#' @import Matrix
#' @importFrom stats rnorm
#'
#' @param fixedSeed determines if they seed should be fixed, which is important
#' for testing
#'
#' @keywords internal
#'
#' @return number of the block processed
imputeMissingDataForBlock <- function(block, dir, epochs, lambda = 1,
                                      gamma = 0.01, r = 10) {
  blockNr <- block$blockNr
  D <- block$block
  flog.info(paste("Impute missing data for block", blockNr, "of", block$total))


  ## check if NA values are contained in the block
  if (any(is.na(D))) {


    ## set D as matrix
    dat <- as.matrix(D)

    ## set NA data to 0
    D <- dat
    D[is.na(D)] <- 0
    D <- as(as.matrix(D), "dgCMatrix")

    Dsummary <- summary(D)
    m <- nrow(D) # number of rows
    n <- ncol(D) # number of columns
    is <- Dsummary$i # row of each revealed entry
    js <- Dsummary$j # column of each revealed entry
    xs <- Dsummary$x # value of each revealed entry
    N <- length(is) # number of revealed entries

    L <- matrix(rnorm(m * r), m, r) / sqrt(r)
    R <- matrix(rnorm(r * n), r, n) / sqrt(r)




    ## run LFM
    resultGd <- runGradientDescent(
      L = L, R = R, lambda = lambda,
      epochs = epochs, gamma = gamma,
      blockNr = blockNr, is = is, js = js,
      D = dat, r = r
    )

    dataTemp <- block$block
    ## Predicted matrix

    D1 <- resultGd$L %*% resultGd$R
    ## Replace predicted values with known, keep predicted unknown
    ## values
    for (i1 in seq_len(nrow(dataTemp)))
      for (j1 in seq_len(ncol(dataTemp)))
      {
        if (!is.na(dataTemp[i1, j1])) {
          D1[i1, j1] <- dataTemp[i1, j1]
        }
      }
  }

  ## no NA values contained in the block - keep original values
  else {
    flog.debug(paste(
      "Block", blockNr,
      "has no missing values. Original values are kept"
    ))
    D1 <- D
  }

  D1 <- as.matrix(D1)
  colnames(D1) <- colnames(D)
  rownames(D1) <- rownames(D)

  filename <- paste("D", blockNr, ".RData", sep = "")
  path <- paste(dir, filename, sep = "/")
  save(D1, file = path)

  
  return(list(blockNr = block$blockNr, block = D1, total = block$total))
}

#' findOutsideValues
#'
#' @aliases findOutsideValues
#'
#' @seealso \code{\link{replaceOutsideValues}}
#' @seealso \code{\link{correctBatchEffect}}
#'
#' @title Find DNA methylation values out of the boundaries
#'
#' @description A method which lists values below 0 or beyond 1 contained in the
#' input matrix. These entries are stored in a data.frame together with the
#' corresponding row and column position of the matrix. Note that this method is
#' especially designed for DNA methylation data.
#'
#' @details Note that this method is especially designed to run after the batch
#' effect correction of DNA methylation data, e.g. with the
#' \code{\link{correctBatchEffect}} method. It can happen, that the predicted values are
#' lying slightly below the lower bound of 0 or beyond the upper bound of 1.
#' This method finds these inaccurately predicted entries. Another method called
#' \code{\link{replaceOutsideValues}} replaces these values either by 0 or 1,
#' respectively.
#'
#' @param data any matrix filled with values that normally should be bounded
#' between 0 and 1.
#'
#' @export findOutsideValues
#' @import futile.logger
#' @usage findOutsideValues(data)
#'
#' @return A data frame containing the columns "level", "row", "col" and "value"
#' defining if the value is below 0 or beyond 1 (level = 0 or level = 1),
#' the row position and the column position in the input matrix and the
#' value itself, respectively.
#'
#' @examples
#' data(BEclearCorrected)
#' # Find predicted values outside of the boundaries
#' outsideEntries <- findOutsideValues(data = ex.corrected.data)
findOutsideValues <- function(data) {
  ## find entries > 1 or < 0 in the matrix, store these entries in a
  ## data.frame
  outsideEntries <- as.data.frame(matrix(nrow = 0, ncol = 4))
  colnames(outsideEntries) <- c("level", "row", "col", "value")
  counter <- 1
  for (i in seq_len(nrow(data))) {
    for (j in seq_len(ncol(data))) {
      if (data[i, j] > 1) {
        outsideEntries[counter, "level"] <- 1
        outsideEntries[counter, "row"] <- i
        outsideEntries[counter, "col"] <- j
        outsideEntries[counter, "value"] <- data[i, j]
        counter <- counter + 1
      }
      if (data[i, j] < 0) {
        outsideEntries[counter, "level"] <- 0
        outsideEntries[counter, "row"] <- i
        outsideEntries[counter, "col"] <- j
        outsideEntries[counter, "value"] <- data[i, j]
        counter <- counter + 1
      }
    }
  }
  flog.info(paste(counter - 1, " values outside of the boundaries found", sep = ""))
  return(outsideEntries)
}

testthat::context("Identification of values below 0 or above 1")

testthat::test_that("Matrix with values above and below", {
  res1 <- findOutsideValues(matrix(c(2, 0.5, 0.4, 0.2, -1, -0.1), ncol = 2))

  testthat::expect_equal(as.numeric(res1[1, ]), c(1, 1, 1, 2))
  testthat::expect_equal(as.numeric(res1[2, ]), c(0, 2, 2, -1.0))
  testthat::expect_equal(as.numeric(res1[3, ]), c(0, 3, 2, -0.1))
})

#' preprocessBEclear
#'
#' @param data any matrix filled with beta values, column names have to be
#' sample_ids corresponding to the ids listed in "samples", row names have to
#' be gene names.
#' @param samples data frame with two columns, the first column has to contain
#' the sample numbers, the second column has to contain the corresponding batch
#' number. Colnames have to be named as "sample_id" and "batch_id".
#' 
#' @description this methods does some preprocessing steps for the later methods
#' like removing rows containing only missing values
#' 
#' @details 
#' Here we describe the preprocessing steps in the order they are executed:
#' \itemize{
#' \item{Values below 0 or above 1 are set to \code{NA}, as the other methods 
#' expect methylation beta values}
#' \item{columns that only contain \code{NA}s are removed}
#' \item{rows that only contain \code{NA}s are removed}
#' \item{samples that are present in the data, but are not annoted in the samples 
#' are removed. If this is the case with your data-set, please check those samples.}
#' \item{samples that are annoted but not in the data matrix are removed}
#' \item{if there are duplicate sample names in the data matrix, all sample names
#' get replaced through a new unique ID. In this case a \code{\link[data.table]{data.table}}
#' containing the mapping is returned as well}
#' }
#'
#' @return a list containing the processed data and samples and a 
#' \code{\link[data.table]{data.table}} containing mappings from the original
#' sample names to the new ones. If sample names weren't changed this third
#' object is \code{NULL}
#' @export
#'
#' @examples
#' data(BEclearData)
#' res <- preprocessBEclear(ex.data, ex.samples)
preprocessBEclear <- function(data, samples) {
    ## checking if they're are values above 1 or below 0
    if (any(data > 1 | data < 0, na.rm = TRUE)) {
        flog.warn(paste(
            sum(data > 1 | data < 0, na.rm = TRUE),
            "values are above 1 or below 0. Check your data"
        ))
        flog.warn("Replacing them with missing values")
        data[data > 1 | data < 0] <- NA
    }
    
    ## checking if there are columns containing only missing values
    naIndices <- apply(data, 2, function(x) all(is.na(x)))
    if (any(naIndices, na.rm = TRUE)) {
        flog.warn("There are columns, that contain only missing values")
        flog.warn(paste(sum(naIndices), "columns get dropped"))
        data <- data[, !naIndices]
    }
    
    ## checking if there are rows containing only missing values
    naIndices <- apply(data, 1, function(x) all(is.na(x)))
    if (any(naIndices, na.rm = TRUE)) {
        flog.warn("There are rows, that contain only missing values")
        flog.warn(paste(sum(naIndices), "rows get dropped"))
        data <- data[!naIndices, ]
    }
    
    ## checking if there are samples that are not present in the samples matrix
    if (any(!colnames(data) %in% samples$sample_id)) {
        ids <- paste(colnames(data)[!colnames(data) %in% samples$sample_id],
                     collapse = ", "
        )
        flog.warn(paste(
            "The following samples are in the data, but not annotated",
            "in the samples matrix:", ids
        ))
        flog.warn("Dropping those samples now")
        data <- data[, colnames(data) %in% samples$sample_id]
    }
    
    if (any(!samples$sample_id %in% colnames(data))) {
        ids <- paste(samples$sample_id[!samples$sample_id %in% colnames(data)],
                     collapse = ", "
        )
        flog.warn(
            "The following samples are annotated in the sample matrix,",
            "but aren't contained in data matrix:", ids
        )
        flog.warn("Dropping those samples now")
        samples <- samples[sample_id %in% colnames(data)]
    }
    
    samples <- data.table(samples)
    uniqueIDsToSamples <- NULL
    
    ## checking if there are duplicated sample names
    if (any(duplicated(colnames(data)))) {
        flog.warn("Sample names aren't unique")
        flog.warn(paste(
            "Transforming them to unique IDs. List with annotations will",
            "be added to the results"
        ))
        uniqueIDsToSamples <- data.table(
            sample_id = colnames(data),
            unique_id =
                as.character(seq_along(colnames(data)))
        )
        colnames(data) <- uniqueIDsToSamples$unique_id
        samples <- samples[uniqueIDsToSamples, ,
                           on = .(sample_id = sample_id)
                           ][
                               ,
                               .(
                                   sample_id = unique_id,
                                   batch_id = batch_id
                               )
                               ]
    }
    setkey(samples, "batch_id", "sample_id")
    
    return(list(data=data, samples = samples, 
                uniqueIDsToSamples = uniqueIDsToSamples ))
}

#' calcBatchEffects
#'
#' @aliases calcBatchEffects
#'
#' @title Calculate the Batch Effects in a given data set
#'
#' @seealso \code{\link{ks.test}}
#' @seealso \code{\link{p.adjust}}
#' @seealso \code{\link{correctBatchEffect}}
#'
#' @description Calculates for each gene in every batch the median distance to the other
#' batches and the p-value resulting from the Kolmogorov-Smirnov test.
#'
#' @details
#' \enumerate{
#' \item{medians}
#' Compares the median value of all beta values belonging to one
#' batch with the median value of all beta values belonging to all other
#' batches. Returns a matrix containing this median difference value for every
#' gene in every batch, columns define the batch numbers, rows the gene names.
#' \item{p-values}
#' Compares the distribution of all beta values corresponding to
#' one batch with the distribution of all beta values corresponding to all
#' other batches and returns a p-value which defines if the distributions are
#' the same or not. Standard two sided Kolmogorov-Smirnov test is used to
#' calculate the (adjusted) p-values.
#' }
#'
#' @param data a \code{\link[data.table]{data.table}} with one column indicating
#' the sample, one the features and a value column indicating the beta value
#' or a matrix with rows as features and columns as samples
#' @param samples data frame with two columns, the first column has to contain
#' the sample numbers, the second column has to contain the corresponding batch
#' number. Colnames have to be named as "sample_id" and "batch_id".
#' @param adjusted should the p-values be adjusted or not, see "method" for
#' available adjustment methods.
#' @param method adjustment method for p-value adjustment (if TRUE), default
#' method is "false discovery rate adjustment", for other available methods
#' see the description of the used standard R package \code{\link{p.adjust}}.
#' @param BPPARAM An instance of the
#' \code{\link[BiocParallel]{BiocParallelParam-class}} that determines how to
#' parallelisation of the functions will be evaluated.
#'
#' @export calcBatchEffects
#' @import futile.logger
#' @import data.table
#' @importFrom stats p.adjust
#' @importFrom methods is
#' @usage calcBatchEffects(data, samples, adjusted=TRUE, method="fdr",
#' BPPARAM=SerialParam())
#'
#' @return a matrix containing medians and p-values for all genes in all batches
#' @examples
#' ## Shortly running example. For a more realistic example that takes
#' ## some more time, run the same procedure with the full BEclearData
#' ## dataset.
#' 
#' ## Calculate fdr-adjusted p-values in non-parallel mode
#' data(BEclearData)
#' ex.data <- ex.data[31:90, 7:26]
#' ex.samples <- ex.samples[7:26, ]
#' 
#' 
#' res <- calcBatchEffects(data = ex.data, samples = ex.samples, method = "fdr")
#' 
#' ## How to handle data-sets without defined batches
#' ## https://github.com/Livia-Rasp/BEclear/issues/22
#' library(data.table)
#' data(BEclearData)
#'
#' DT <- data.table(ex.samples)[, .(sample_id)]
#' 
#' ## set the batch_id equal to the sample_id
#' ## this way samples are treated as batches
#' DT[, batch_id := sample_id]
#' 
#' res <- calcBatchEffects(data = ex.data, samples = DT)
calcBatchEffects <- function(data, samples, adjusted = TRUE, method = "fdr",
                             BPPARAM = SerialParam()) {
  if (!is(data, "data.table")) {
    flog.info("Transforming matrix to data.table")
    data <- data.table(feature = as.character(rownames(data)), data)
    data <- melt(
      data = data, id.vars = "feature", variable.name = "sample",
      value.name = "beta.value", variable.factor = FALSE
    )
    setkey(data, "feature", "sample")
  }

  if (!is(samples, "data.table")) {
    samples <- data.table(samples)
  }

  flog.info(paste(
    "Calculate the batch effects for",
    samples[, uniqueN(batch_id)], "batches"
  ))

  batchEffects <- lapply(samples[, unique(batch_id)], calcBatchEffectsForBatch,
    samples = samples, data = data, BPPARAM = BPPARAM)

  flog.debug(paste("Binding", length(batchEffects), "columns of batch_effects together"))
  batchEffects <- do.call(cbind, batchEffects)

  med <- batchEffects[colnames(batchEffects) == "medians"]
  colnames(med) <- samples[, unique(batch_id)]

  pvalues <- batchEffects[colnames(batchEffects) == "p_values"]
  colnames(pvalues) <- samples[, unique(batch_id)]

  # pvalue adjustment
  if (adjusted == TRUE) {
    flog.info("Adjusting p-values")
    pvalues <- t(apply(pvalues, 1, p.adjust, method = method))
  }

  return(list(med = med, pval = pvalues))
}

testthat::context("Testing the calculation of BE scores")

testthat::test_that("matrix with batch effects", {
  data(BEclearData)
  ex.data <- ex.data[31:90, 7:26]
  ex.samples <- ex.samples[7:26, ]
  batchEffects <- calcBatchEffects(data = ex.data, samples = ex.samples,
                                    adjusted = TRUE, method = "fdr")
  med <- batchEffects$med
  pvals <- batchEffects$pval
  
  
  sum <- calcSummary(medians = med, pvalues = pvals)
  score.table <- calcScore(
    data = ex.data, samples = ex.samples, summary = sum,
    saveAsFile = FALSE
  )

  testthat::expect_equal(score.table$BEscore, c(0, 0, 0, 0.73333333))
})

#' calcScore
#'
#' @aliases calcScore
#'
#' @title calculate batch effect score
#'
#' @seealso \code{\link{calcBatchEffects}}
#' @seealso \code{\link{calcSummary}}
#' @seealso \code{\link{correctBatchEffect}}
#'
#' @description Returns a table with the number of found genes with found
#' p-values less or equal to 0.01 and median values greater or equal to 0.05.
#' A score is calculated depending on the number of found genes as well as the
#' magnitude of the median difference values, this score is divided by the
#' overall number of genes in the data and returned as "BEscore". See details
#' for further information and details about the score calculation. The
#' returned data.frame is also stored in the specified directory as .RData file.
#'
#' @details The returned data frame contains one column for the batch numbers,
#' 11 columns containing the number of genes found in a certain range of the
#' median difference value and a column with the calculated BEscore. These
#' found genes are assumed to be batch affected due to their difference in
#' median values and their different distribution of the beta values. The higher
#' the found number of genes and the more extreme the median difference is, the
#' more severe is the assumed batch effect supposed to be. We suggest that there
#' is no need for a batch effect correction if the BEscore for a batch is less
#' than 0.02. BEscores between 0.02 and 0.1 are lying in a "gray area" for which
#' we assume a not severe batch effect, and values beyond 0.1 certainly describe
#' a batch effect and should definitely be corrected.\cr
#' The 11 columns containing the numbers of found genes count the median
#' difference values which are ranging from >= 0.05 to < 0.1 ; >= 0.1 to < 0.2;
#' >= 0.2 to < 0.3 and so on up to a limit of 1.\cr
#' The BEscore is calculated by the sum of the weighted number of genes divided
#' by the number of genes. Weightings are calculated by multiplication of the
#' number of found genes between 0.05 and 0.1 by 1, between 0.1 and 0.2 by 2,
#' between 0.2 and 0.3 by 4, between 0.3 and 0.4 by 6 and so on.
#'
#' @param data any matrix filled with beta values, column names have to be
#' sample_ids corresponding to the ids listed in "samples", row names have to
#' be gene names.
#' @param samples data frame with two columns, the first column has to contain
#' the sample numbers, the second column has to contain the corresponding batch
#' number. Colnames have to be named as "sample_id" and "batch_id".
#' @param summary a summary \code{\link[data.table]{data.table}} containing the 
#' columns "gene", "batch", median" and "p-value" and consists of all genes which 
#' were found in the median and p-value calculations, see 
#' \code{\link{calcSummary}} function for more details.
#' @param saveAsFile determining if the data.frame should also be saved as a file
#' @param dir set the path to a directory the returned data.frame should be
#' stored. The current working directory is defined as default parameter.
#'
#' @export calcScore
#' @import futile.logger
#' @import data.table
#' @importFrom dixonTest dixonTest
#' @usage calcScore(data, samples, summary, saveAsFile=FALSE, dir=getwd())
#' 
#' @references \insertRef{Dixon1950}{BEclear}
#' @references \insertRef{Dixon1951}{BEclear}
#' @references \insertRef{Rorabacher1991}{BEclear}
#'
#' @return A data.frame is returned containing the number of found genes assumed
#' to be batch affected separated by batch and a BEscore for every batch. Furthermore
#' there's a column dixonPval giving you a p-value regarding each BEscore according 
#' to a Dixon test.
#' The data.frame is also stored in the specified directory as .RData file, if 
#' saveAsFile is \code{TRUE}.
#'
#' @examples
#' ## Shortly running example. For a more realistic example that takes
#' ## some more time, run the same procedure with the full BEclearData
#' ## dataset.
#' 
#' ## Whole procedure that has to be done to use this function.
#' data(BEclearData)
#' ex.data <- ex.data[31:90, 7:26]
#' ex.samples <- ex.samples[7:26, ]
#' ## Calculate the batch effects
#' batchEffects <- calcBatchEffects(data = ex.data, samples = ex.samples,
#' adjusted = TRUE, method = "fdr")
#' med <- batchEffects$med
#' pvals <- batchEffects$pval
#' 
#' # Summarize p-values and median differences for batch affected genes
#' sum <- calcSummary(medians = med, pvalues = pvals)
#' 
#' # Calculates the score table
#' score.table <- calcScore(data = ex.data, samples = ex.samples, summary = sum)
calcScore <- function(data, samples, summary, saveAsFile = FALSE, dir = getwd()) {
  ## take batch ids
  batches <- unique(samples$batch_id)
  flog.info(paste("Calculating the scores for", length(batches), "batches"))
  
  ## take number of genes
  numGenes <- nrow(data)
  
  ## count the medians
  DT <- lapply(batches, FUN = function(x, y){
      data.table(batch_id = x, 
                 count05 = y[batch_id == x & median >= 0.05 & median < 0.1, .N], 
                 count1 = y[batch_id == x & median  >= 0.1  & median < 0.2,  .N], 
                 count2 = y[batch_id == x & median  >= 0.2  & median < 0.3,  .N], 
                 count3 = y[batch_id == x & median  >= 0.3  & median < 0.4,  .N],
                 count4 = y[batch_id == x & median  >= 0.4  & median < 0.5,  .N], 
                 count5 = y[batch_id == x & median  >= 0.5  & median < 0.6,  .N], 
                 count6 = y[batch_id == x & median  >= 0.6  & median < 0.7,  .N], 
                 count7 = y[batch_id == x & median  >= 0.7  & median < 0.8,  .N], 
                 count8 = y[batch_id == x & median  >= 0.8  & median < 0.9,  .N], 
                 count9 = y[batch_id == x & median  >= 0.9,  .N])}, summary)
  
  DT <- do.call(rbind, DT)
  
  ## calculate BEscores
  DT[, BEscore := (count05 * 1 + count1* 2 + count2 * 4 + count3 * 6 + count4 * 8 
                   + count5 * 10 + count6 * 12 + count7 * 14 + count8 * 16 
                   + count9 * 18)/numGenes ]
  
  
  
  ## calculate outlier according to the dixon test
  DT[, dixonPval := as.numeric(NA)]
  if(sum(!is.na(DT$BEscore)) >= 3 && max(DT$BEscore) != min(DT$BEscore)){
    pval <- dixonTest(DT$BEscore, alternative = c("two.sided"))
    DT[BEscore == max(DT$BEscore), dixonPval := as.numeric(pval$p.value)]
  }
  
  
  
  if (saveAsFile) {
      filename <- "score.table.Rdata"
      save(DT, file = paste(dir, filename, sep = "/"))
  }
  
  return(DT)
}

#' calcBatchEffectsForBatch
#'
#' @keywords internal
#'
#' @description  function to calculate batch effects for every gene in a batch
#'
#' @importFrom stats ks.test
#' @import BiocParallel
#' @import futile.logger
#'
#' @return the medians p-values for genes in a batch
calcBatchEffectsForBatch <- function(batch, samples, data, 
                                     BPPARAM = SerialParam()) {
  flog.debug(paste("Calculating the batch effect for batch", batch))

  flog.debug("Subsetting data.table for batch")
  DT_batch <- samples[batch_id == batch][data
    , ,
    on = .(sample_id = sample),
    nomatch = 0
  ][, .(feature, beta.value)]
  flog.debug("Subsetting data.table for others")
  DT_other <- samples[batch_id != batch][data
    , ,
    on = .(sample_id = sample),
    nomatch = 0
  ][, .(feature, beta.value)]


  features_batch <- DT_batch[, list(list(beta.value)), by = feature]$V1
  features_other <- DT_other[, list(list(beta.value)), by = feature]$V1

  flog.debug(paste("Calculating the p-values for batch", batch))
  result <- bpmapply(function(X, Y) {
    medianDiff <- median(X, na.rm = TRUE) - median(Y, na.rm = TRUE)
    if (all(is.na(X))) {
      return(c(0.0, medianDiff))
    } else {
      suppressWarnings(pval <- ks.test(X, Y)$p.value)
      return(c(pval, medianDiff))
    }
  }, X = features_batch, Y = features_other, BPPARAM = BPPARAM)

  result <- t(result)
  colnames(result) <- c("p_values", "medians")

  return(data.frame(result, row.names = unique(data$feature)))
}

testthat::context("Testing the caculations of batch effects for single batch")

testthat::test_that("2 batches, only available values", {
  data <- data.table(
    sample = c(1, 1, 1, 2, 2, 2, 3, 3, 3), feature = c(1, 2, 3, 1, 2, 3, 1, 2, 3),
    beta.value = c(
      0.29382666, 0.08099936, 0.18366184,
      0.16625504, -1.26959907, 2.34949332,
      -1.41200541, -0.01696149, -0.54431935
    )
  )

  samples <- data.table(sample_id = c(1, 2, 3), batch_id = c(1, 1, 2))

  res1 <- calcBatchEffectsForBatch(batch = 1, samples = samples, data = data,
                                   BPPARAM = SerialParam())
  res2 <- calcBatchEffectsForBatch(batch = 2, samples = samples, data = data,
                                   BPPARAM = SerialParam())

  testthat::expect_equal(res1[, 1], c(0.667, 1.000, 0.667), tolerance = .001)
  testthat::expect_equal(res2[, 1], c(0.667, 1.000, 0.667), tolerance = .001)
  testthat::expect_equal(res1[, 2], c(1.642, -0.577, 1.811), tolerance = .001)
  testthat::expect_equal(res2[, 2], c(-1.642, 0.577, -1.811), tolerance = .001)
})

testthat::test_that("3 batches, NAs", {
  data <- data.table(
    sample = c(1, 1, 1, 2, 2, 2, 3, 3, 3, 4, 4, 4, 5, 5, 5),
    feature = c(1, 2, 3, 1, 2, 3, 1, 2, 3, 1, 2, 3, 1, 2, 3),
    beta.value = c(
      NA, 0.08099936, 0.18366184,
      0.16625504, -1.26959907, NA,
      -1.41200541, -0.01696149, -0.54431935,
      -0.6548353, NA, -1.2723697,
      0.5380646, NA, -1.1657070
    )
  )

  samples <- data.table(sample_id = c(1, 2, 3, 4, 5), batch_id = c(1, 1, 2, 3, 3))

  res1 <- calcBatchEffectsForBatch(batch = 1, samples = samples, data = data,
                                   BPPARAM = SerialParam())
  res2 <- calcBatchEffectsForBatch(batch = 2, samples = samples, data = data,
                                   BPPARAM = SerialParam())
  res3 <- calcBatchEffectsForBatch(batch = 3, samples = samples, data = data,
                                   BPPARAM = SerialParam())

  testthat::expect_equal(res1[, 1], c(1.00, 1.00, 0.50), tolerance = .001)
  testthat::expect_equal(res2[, 1], c(0.50, 1.00, 1.00), tolerance = .001)
  testthat::expect_equal(res3[, 1], c(1.00, 0.00, 0.333), tolerance = .001)

  testthat::expect_equal(res1[, 2], c(0.821, -0.577, 1.349), tolerance = .001)
  testthat::expect_equal(res2[, 2], c(-1.578, 0.577, 0.621), tolerance = .001)
  testthat::expect_equal(res3[, 2], c(0.564, NA, -1.039), tolerance = .001)
})

#' runGradientDescent
#'
#' @keywords internal
#'
#' @description Runner for gradient descent (or stochastic gradient descent) for
#' the specified number of epoch
#'
#' @return a list containing two matrices generated by the gradient descent
runGradientDescent <- function(L, R, lambda, epochs, gamma = 0.01, blockNr,
                               is, js, D, r) {
  # flog.debug(paste("Calculating the gradient descent for block", blockNr))
  LR <- list(L = L, R = R)

  loss_result <- loss(LR$L, LR$R, lambda, D = D)

  curLoss <- loss_result$loss
  error_matrix <- loss_result$error_matrix

  for (epoch in seq_len(epochs)) {
    # flog.debug(paste("Calculating gradient descent epoch", epoch, "of",
    #                 epochs, "for block", blockNr))
    LR <- gdepoch(LR$L, LR$R, lambda, gamma,
      is = is, js = js, D = D,
      error_matrix = error_matrix
    )

    ## bold driver step size update
    oldLoss <- curLoss

    loss_result <- loss(LR$L, LR$R, lambda, D = D)

    curLoss <- loss_result$loss
    
    error_matrix <- loss_result$error_matrix
    
    if(is.infinite(curLoss)){
      flog.error("The Gradient Descent diverged.")
      flog.error("This indicates that the step size is to big.")
      flog.error("Try lowering gamma to fix this.")
      stop("The Gradient Descent diverged")
    }
    
    if (oldLoss < curLoss) {
      gamma <- gamma / 2
    } else {
      gamma <- gamma * 1.05
    }
  }
  return(LR)
}

testthat::context("Testing the gradient descent for a multiple epochs")

testthat::test_that("quadratic D matrix, only integer values given", {
  res1 <- runGradientDescent(
    L = matrix(c(1, 2)), R = t(matrix(c(3, 4))),
    D = matrix(c(1, 1, 1, 1), nrow = 2),
    lambda = 0.2, is = c(1, 2, 1, 2), js = c(1, 1, 2, 2),
    gamma = 0.01, r = 1, epochs = 5, block = 1
  )

  testthat::expect_equal(res1$L, matrix(c(0.347, 0.393)), tolerance = .01)
  testthat::expect_equal(res1$R, t(matrix(c(2.61, 3.43))), tolerance = .01)
})

testthat::test_that("long D matrix, only integer values given", {
  res1 <- runGradientDescent(
    L = matrix(c(5, 6)), R = t(matrix(c(7, 4, 8))),
    D = matrix(c(7, 5, 3, 1, 3, 4), nrow = 2),
    lambda = 0.2, is = c(1, 2, 1, 2, 1, 2), js = c(1, 1, 2, 2, 3, 3),
    gamma = 0.01, r = 1, epochs = 5, block = 1
  )

  testthat::expect_equal(res1$L, matrix(c(-6.12, -7.76)), tolerance = .01)
  testthat::expect_equal(res1$R, t(matrix(c(-0.831, -0.266, -0.504))), tolerance = .01)
})

testthat::test_that("wide D matrix, only integer values given", {
  res1 <- runGradientDescent(
    L = matrix(c(5, 1, 2)), R = t(matrix(c(3, 3))),
    D = matrix(c(5, 6, 10, 11, 3, 1), nrow = 3),
    lambda = 0.2, is = c(1, 2, 3, 1, 2, 3), js = c(1, 1, 1, 2, 2, 2),
    gamma = 0.01, r = 1, epochs = 5, block = 1
  )

  testthat::expect_equal(res1$L, matrix(c(3.73, 1.68, 2.20)), tolerance = .01)
  testthat::expect_equal(res1$R, t(matrix(c(2.26, 2.26))), tolerance = .01)
})

#' gdepoch
#'
#' @keywords internal
#'
#' @description Run a gradient descent epoch (L and R are starting points,
#' gamma is stgammaize)
#'
#' @return a list containing two matrices generated by the gradient descent for
#' the current epoch
gdepoch <- function(L, R, lambda, gamma, is, js, D,
                    error_matrix = NULL) {
  if (is.null(error_matrix)) {
    error_matrix <- (D - L %*% R)
  }

  locLoss <- localLoss(L, R, is, js, error_matrix)

  ## perform a gradient step on L and R with step size gamma
  ## by using the gradient matrices


  L <- L + gamma * 2 * (locLoss$dL - lambda * L)
  R <- R + gamma * 2 * (locLoss$dR - lambda * R)



  ## return result
  return(list(L = L, R = R))
}

#' countValuesToPredict
#'
#' @aliases countValuesToPredict
#'
#' @seealso \code{\link{clearBEgenes}}
#' @seealso \code{\link{correctBatchEffect}}
#' @seealso \code{\link{correctBatchEffect}}
#'
#' @title Count NA entries in a matrix
#'
#' @description Simple function that counts all values in a matrix which are NA
#'
#' @return Returns a data frame with the number of NA entries per column. Since
#' the function is mainly written for the usage in batch effect correction of
#' DNA methylation data, the column names of the data frame are set to "sample"
#' and "num_pred_values". Nevertheless, the function can be used with any other
#' matrix containing anything but beta values.
#'
#' @param data any kind of matrix
#'
#' @export countValuesToPredict
#' @import futile.logger
#' @usage countValuesToPredict(data)
#'
#' @examples
#' ## Shortly running example. For a more realistic example that takes
#' ## some more time, run the same procedure with the full BEclearData
#' ## dataset
#' 
#' ## Whole procedure that has to be done to use this function.
#' data(BEclearData)
#' ex.data <- ex.data[31:90, 7:26]
#' ex.samples <- ex.samples[7:26, ]
#' 
#' ## Calculate the batch effects
#' batchEffects <- calcBatchEffects(data = ex.data, samples = ex.samples,
#' adjusted = TRUE, method = "fdr")
#' meds <- batchEffects$med
#' pvals <- batchEffects$pval
#' 
#' ## Summarize p-values and median differences for batch affected genes
#' sum <- calcSummary(medians = meds, pvalues = pvals)
#' clearedMatrix <- clearBEgenes(data = ex.data, samples = ex.samples, summary = sum)
#' numberOfEntries <- countValuesToPredict(data = clearedMatrix)
countValuesToPredict <- function(data) {
  amount <- sum(is.na(data)) / (nrow(data) * ncol(data))
  flog.info(paste(sum(is.na(data)), "values (", amount * 100,
    "% of the data) set to NA",
    sep = " "
  ))
  numberPredValues <- matrix(ncol = 2, nrow = ncol(data))
  numberPredValues <- as.data.frame(numberPredValues)
  colnames(numberPredValues) <- c("sample", "num_pred_values")
  numberPredValues[, 1] <- colnames(data)
  numberPredValues[, 2] <- 0
  for (i in seq_len(nrow(data))) {
    for (j in seq_len(ncol(data))) {
      if (is.na(data[i, j])) {
        numberPredValues[numberPredValues$sample == j, 2] <- numberPredValues[numberPredValues$sample == j, 2] + 1
      }
    }
  }
  return(numberPredValues = numberPredValues)
}

#' calcSummary
#'
#' @aliases calcSummary
#'
#' @title Summarize median comparison and p-value calculation results
#'
#' @seealso \code{\link{calcBatchEffects}}
#' @seealso \code{\link{correctBatchEffect}}
#'
#' @description  Summarizes the results of the \code{\link{calcBatchEffects}} 
#' function
#'
#' @details All genes with a median comparison value >= 0.05 and a p-value
#' of <= 0.01 are summarized into a data.frame. These genes are assumed to
#' contain a batch effect
#'
#' @param medians a matrix containing median difference values calculated by
#' the \code{\link{calcBatchEffects}} function. For further details look at the
#' documentation of this function.
#' @param pvalues a matrix containing p-values calculated by the
#' \code{\link{calcBatchEffects}} function. For further details look at the
#' documentation of this function.
#' @param mediansTreshold the threshold above or equal median values are regarded 
#' as batch effected, when the criteria for p-values is also met.
#' @param pvaluesTreshold the threshold below or equal p-values are regarded as 
#' batch effected, when the criteria for medians is also met.
#'
#' @export calcSummary
#' @importFrom abind abind
#' @importFrom futile.logger flog.info
#' @import data.table
#' @usage calcSummary(medians, pvalues, mediansTreshold, pvaluesTreshold)
#'
#' @return Null if there are no batch effects detected, else
#' a \code{\link[data.table]{data.table}} with the columns "gene" containing the gene name,
#' "batch" containing the batch number from which the gene was found, "median"
#' and "p-value" containing the calculated median difference values and the
#' p-values, respectively.
#'
#' @examples
#' ## Shortly running example. For a more realistic example that takes
#' ## some more time, run the same procedure with the full BEclearData
#' ## dataset.
#' 
#' ## Whole procedure that has to be done to use this function.
#' data(BEclearData)
#' ex.data <- ex.data[31:90, 7:26]
#' ex.samples <- ex.samples[7:26, ]
#' 
#' ## Calculate the batch effects
#' batchEffects <- calcBatchEffects(data = ex.data, samples = ex.samples,
#' adjusted = TRUE, method = "fdr")
#' med <- batchEffects$med
#' pvals <- batchEffects$pval
#' 
#' ## Summarize p-values and median differences for batch affected genes
#' sum <- calcSummary(medians = med, pvalues = pvals)
calcSummary <- function(medians, pvalues, mediansTreshold = 0.05, 
                        pvaluesTreshold = 0.01) {
    
    flog.info("Generating a summary table")
    x <- abind(medians, pvalues, along = 3)
    ## find affected genes
    affected <- apply(x, c(1,2), FUN = 
                          function(X){ return((is.na(X[1]) |  X[1] >= mediansTreshold) 
                                   & X[2] <= pvaluesTreshold)})
    
    if(sum(affected) == 0){
        return(NULL)
    }else{
        tmp <- arrayInd(which(affected), dim(affected))
        
        return(data.table(gene = rownames(medians)[tmp[,1]],
                          batch_id = colnames(medians)[tmp[,2]],
                          median = medians[affected], 
                          pvalue = pvalues[affected]))
    }
}

testthat::context("Testing the summary calculation")

testthat::test_that("matrix with batch effects", {
  sum <- calcSummary(
    data.frame(
      b1 = c(0.9, NA, 0.4), b2 = c(NA, 0.001, 0.8),
      b3 = c(0.2, 0.5, 0.01)
    ),
    data.frame(
      b1 = c(0.9, 0.3, 0.04), b2 = c(0.01, 0.002, 0.06),
      b3 = c(0.9, 0.00001, 0.4)
    )
  )
  testthat::expect_equal(sum$median, c(NA, 0.5))
  testthat::expect_equal(sum$pvalue, c(0.01, 0.00001))
})

testthat::test_that("matrix without batch effects", {
  sum <- calcSummary(
    data.frame(
      b1 = c(0.1, 0.2, 0.4), b2 = c(0.01, 0.001, 0.8),
      b3 = c(0.2, 0.5, 0.01)
    ),
    data.frame(
      b1 = c(0.9, 0.3, 0.04), b2 = c(0.01, 0.002, 0.06),
      b3 = c(0.9, 0.1, 0.4)
    )
  )

  testthat::expect_null(sum)
})

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#' Calculating the Local Loss
#' 
#' @keywords internal
#' 
#' @description Calculates the gradient dR and dL for the current epoch.
#' 
#' @return a list containing two matrices containing the local loss
#'
#' @import Rcpp
#' @export localLoss
localLoss <- function(L, R, is, js, error_matrix) {
    .Call('_BEclear_localLoss', PACKAGE = 'BEclear', L, R, is, js, error_matrix)
}

testthat::context("Testing the local loss")

testthat::test_that("quadratic error matrix, only integer values given", {
  res1 <- localLoss(
    L = matrix(c(1, 2)), R = t(matrix(c(3, 4))),
    error_matrix = matrix(c(1, 1, 1, 1), nrow = 2),
    is = c(1, 2, 1, 2), js = c(1, 1, 2, 2)
  )

  testthat::expect_equal(res1$dL, matrix(c(7, 7)), tolerance = .01)
  testthat::expect_equal(res1$dR, t(matrix(c(3, 3))), tolerance = .01)
})

testthat::test_that("quadratic error matrix, only integer values given, r =3", {
  res1 <- localLoss(
    L = matrix(c(1, 2, 5, 9, 1, 4), ncol = 3), R = t(matrix(c(3, 4, 9, 2, 6, 7), ncol = 3)),
    error_matrix = matrix(c(1, 17, 2, 41), nrow = 2),
    is = c(1, 2, 1, 2), js = c(1, 1, 2, 2)
  )

  testthat::expect_equal(res1$dL, matrix(c(11, 215, 13, 235, 20, 389), ncol = 3), tolerance = .01)
  testthat::expect_equal(res1$dR, t(matrix(c(35, 84, 158, 379, 69, 166), ncol = 3)), tolerance = .01)
})

testthat::test_that("long error matrix, only integer values given", {
  res1 <- localLoss(
    L = matrix(c(5, 6)), R = t(matrix(c(7, 4, 8))),
    error_matrix = matrix(c(7, 5, 3, 1, 3, 4), nrow = 2),
    is = c(1, 2, 1, 2, 1, 2), js = c(1, 1, 2, 2, 3, 3)
  )

  testthat::expect_equal(res1$dL, matrix(c(85, 71)), tolerance = .01)
  testthat::expect_equal(res1$dR, t(matrix(c(65, 21, 39))), tolerance = .01)
})

testthat::test_that("wide error matrix, only integer values given", {
  res1 <- localLoss(
    L = matrix(c(5, 1, 2)), R = t(matrix(c(3, 3))),
    error_matrix = matrix(c(5, 6, 10, 11, 3, 1), nrow = 3),
    is = c(1, 2, 3, 1, 2, 3), js = c(1, 1, 1, 2, 2, 2)
  )

  testthat::expect_equal(res1$dL, matrix(c(48, 27, 33)), tolerance = .01)
  testthat::expect_equal(res1$dR, t(matrix(c(51, 60))), tolerance = .01)
})

## ----echo = FALSE, message = FALSE--------------------------------------------
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
library(pander)
panderOptions('knitr.auto.asis', TRUE)
panderOptions('plain.ascii', TRUE)

## ----eval=FALSE---------------------------------------------------------------
# if (!requireNamespace("BiocManager", quietly = TRUE)) {
#   install.packages("BiocManager")
# }
# BiocManager::install("BEclear")

## ----eval=FALSE---------------------------------------------------------------
# if (!requireNamespace("devtools", quietly = TRUE)) {
#   install.packages("devtools")
# }
# devtools::install_github("uds-helms/BEclear")

## -----------------------------------------------------------------------------
library(BEclear)

## ----data---------------------------------------------------------------------
data("BEclearData")

## -----------------------------------------------------------------------------
knitr::kable(ex.data[1:10,1:5], caption = 'Some entries from the example data-set')

## -----------------------------------------------------------------------------
knitr::kable(ex.samples[1:10,], caption = 'Some entries from the example sample annotation')

## ----detection, cache=TRUE----------------------------------------------------
batchEffect <- calcBatchEffects(
  data = ex.data, samples = ex.samples,
  adjusted = TRUE, method = "fdr"
)
mdifs <- batchEffect$med
pvals <- batchEffect$pval

## ----summary, cache=TRUE------------------------------------------------------
summary <- calcSummary(medians = mdifs, pvalues = pvals)
knitr::kable(head(summary), caption = 'Summary over the batch affected gene-sample combination of the example data set')

## ----score, cache=TRUE--------------------------------------------------------
score <- calcScore(ex.data, ex.samples, summary, dir = getwd())
knitr::kable(score, caption = 'Batch scores of the example data-set')

## ----clearBE, cache=TRUE------------------------------------------------------
cleared.data <- clearBEgenes(ex.data, ex.samples, summary)

## ----imputation, cache=TRUE---------------------------------------------------
library(ids)
corrected.data <- imputeMissingData(cleared.data,
  rowBlockSize = 60,
  colBlockSize = 60, epochs = 50,
  outputFormat = "", dir = getwd()
)

## ----replace, cache=TRUE------------------------------------------------------
corrected.data.valid<-replaceOutsideValues(corrected.data)

## ----correction, cache=TRUE---------------------------------------------------
result <- correctBatchEffect(data = ex.data, samples = ex.samples)

## -----------------------------------------------------------------------------
?BiocParallel::BiocParallelParam

## ----boxplot1, fig.wide = TRUE, fig.cap = "Distribution of the example beta values grouped by sample"----
makeBoxplot(ex.data, ex.samples, score,
  bySamples = TRUE,
  col = "standard", main = "Example data", xlab = "Batch",
  ylab = "Beta value", scoreCol = TRUE)

## ----boxplot2, fig.wide = TRUE, fig.cap = "Distribution of the corrected beta values grouped by sample"----
makeBoxplot(corrected.data, ex.samples, score,
  bySamples = TRUE,
  col = "standard", main = "Corrected example data",
  xlab = "Batch", ylab = "Beta value", scoreCol = FALSE)

## ----sessionInfo, echo=FALSE--------------------------------------------------
pander(sessionInfo(), compact=TRUE)

#' calcPositions
#'
#' @keywords internal
#'
#' @return  returns vector with number of blocks, start - and stop position of
#' last 2 blocks
#'
calcPositions <- function(num, blockSize) {
  if (blockSize <= 0) {
    stop("blockSizes must be greater than 0")
  }

  if (num <= 0) {
    stop("number of entries must be greater than 0")
  }

  rest <- num %% blockSize
  ## normal start and stop position if nrow can be divided by blockSize
  ## without a rest
  startPos1 <- num - (2 * blockSize) - rest + 1
  startPos2 <- num - blockSize - rest + 1
  stopPos1 <- startPos2 - 1
  stopPos2 <- num
  ## special case if matrix has < blockSize rows
  if (num < blockSize) {
    startPos1 <- 0
    startPos2 <- 1
    stopPos1 <- 0
    blocks <- 1
  } else {
    ## num can be divided by blockSize
    if ((num %% blockSize) == 0) {
      blocks <- num / blockSize # done
    } else {
      ## case not dividable by 100
      ## case rest < 75% of block size
      if (rest < floor(blockSize * 0.75)) {
        blocks <- num %/% blockSize # done
      } else {
        ## case rest > 75%
        y <- blockSize + rest
        blocks <- num %/% blockSize + 1 # done
        ## case y dividable by 2
        if (y %% 2 == 0) {
          startPos1 <- num - y + 1
          startPos2 <- startPos1 + (y %/% 2) + 1
          stopPos1 <- startPos2 - 1
        }
        else {
          startPos1 <- num - y + 1
          startPos2 <- startPos1 + (y %/% 2) + 1
          stopPos1 <- startPos2 - 1
        }
      }
    }
  }
  positions <- c(blocks, startPos1, startPos2, stopPos1, stopPos2)
  names(positions) <- c("numBlocks", "startN-1", "startN", "stopN-1", "stopN")
  return(positions)
}

testthat::context("Calculation of block positions")

testthat::test_that("Values that add up", {
  testthat::expect_equal(unname(calcPositions(100, 10)), c(10, 81, 91, 90, 100))
  testthat::expect_equal(unname(calcPositions(27, 3)), c(9, 22, 25, 24, 27))
  testthat::expect_equal(unname(calcPositions(16, 4)), c(4, 9, 13, 12, 16))
})


testthat::test_that("Values that don't add up", {
  testthat::expect_equal(unname(calcPositions(99, 10)), c(10, 81, 91, 90, 99))
  testthat::expect_equal(unname(calcPositions(200, 13)), c(15, 170, 183, 182, 200))
  testthat::expect_equal(unname(calcPositions(1363, 13)), c(105, 1340, 1353, 1352, 1363))
})

testthat::test_that("Block size smaller than 1", {
  testthat::expect_error(calcPositions(5, -1))
  testthat::expect_error(calcPositions(33, 0))
  testthat::expect_error(calcPositions(1, -3))
})

testthat::test_that("Number of entries smaller than 1", {
  testthat::expect_error(calcPositions(0, 1))
  testthat::expect_error(calcPositions(0, 0))
  testthat::expect_error(calcPositions(-1, -3))
  testthat::expect_error(calcPositions(-20, 67))
})

testthat::test_that("Number smallerthan block size", {
  testthat::expect_equal(unname(calcPositions(13, 55)), c(1, 0, 1, 0, 13))
})

#' makeBoxplot
#'
#' @aliases makeBoxplot
#'
#' @seealso \code{\link{calcScore}}
#' @seealso \code{\link{boxplot}}
#' @seealso \code{\link{correctBatchEffect}}
#'
#' @keywords hplot
#'
#' @title produce simple predefined boxplot for methylation data
#'
#' @description A simple \code{\link{boxplot}} is done with boxes either
#' separated by batches or by samples and describe the five number summary of
#' all beta values corresponding to a batch or a sample, respectively. The
#' batch_ids are shown on the x-axis with a coloring corresponding to the
#' BEscore.
#'
#' @details The color code for the batch_ids on the x-axis provides a simple
#' "traffic light" the user can use to decide if he wants to correct for an
#' assumed batch effect or not. Green means no batch effect, yellow a possibly
#' existing not severe batch effect and red stands for an obviously existing
#' batch effect that should be corrected. The traffic light colors are set
#' according to the BEscore from the \code{\link{calcScore}} function, values
#' from 0 to 0.02 are colored in green, from 0.02 to 0.1 in yellow and values
#' over 0.1 are colored in red.
#'
#' @param data any matrix filled with beta values, column names have to be
#' sample_ids corresponding to the ids listed in "samples", row names have to
#' be gene names.
#' @param samples data frame with two columns, the first column has to contain
#' the sample numbers, the second column has to contain the corresponding batch
#' number. Colnames have to be named as "sample_id" and "batch_id".
#' @param score data frame produced by the \code{\link{calcScore}} function.
#' Contains the number of presumably batch affected genes and a BEscore which
#' is needed for the coloring of the batch_ids.
#' @param bySamples should the boxes be separated by samples or not. If not,
#' boxes are separated by the batch_ids.
#' @param col colors for the boxes, refers to the standard \code{\link{boxplot}}
#' R-function. If it is set to "standard", boxes are colored batch-wise (if
#' separated by samples) or the standard color "yellow" is used (if separated
#' by batches).
#' @param main main title for the box plot. Default is an empty string.
#' @param xlab label for the x-axis of the box plot. Default is "Batch".
#' @param ylab label for the y-axis of the box plot. Default is "Beta value".
#' @param scoreCol should the batch_ids on the a-axis be colored according to
#' the BEscore or not? If not, black is used as color for all batch_ids.
#' @param log TRUE, if the y-axis should be on a logarithmic scale. 
#'
#' @export makeBoxplot
#' @importFrom graphics boxplot mtext par
#' @importFrom methods as
#' @importFrom stats ks.test median rnorm
#' @importFrom utils write.table
#' @usage makeBoxplot(data, samples, score, bySamples=FALSE, col="standard",
#' main="", xlab="Batch", ylab="Beta value", scoreCol=TRUE, log = FALSE)
#'
#' @return Returns a boxplot on the graphic device with the features explained
#' above.
#'
#' @examples
#' ## Shortly running example. For a more realistic example that takes
#' ## some more time, run the same procedure with the full BEclearData
#' ## dataset.
#' 
#' ## Whole procedure that has to be done to use this function.
#' data(BEclearData)
#' ex.data <- ex.data[31:90, 7:26]
#' ex.samples <- ex.samples[7:26, ]
#' 
#' ## Prepare the data for the box plots
#' ## Calculate the batch effects
#' batchEffects <- calcBatchEffects(data = ex.data, samples = ex.samples,
#' adjusted = TRUE, method = "fdr")
#' meds <- batchEffects$med
#' pvals <- batchEffects$pval
#' 
#' ## Summarize p-values and median differences for batch affected genes
#' sum <- calcSummary(medians = meds, pvalues = pvals)
#' 
#' # Calculate the BEscore for the batch_id colorings of the x-axis
#' score <- calcScore(data = ex.data, samples = ex.samples, summary = sum)
#' 
#' ## Simple boxplot for the example data separated by batch
#' makeBoxplot(
#'   data = ex.data, samples = ex.samples, score = score, bySamples = FALSE,
#'   main = "Some box plot"
#' )
#' 
#' ## Simple boxplot for the example data separated by samples
#' makeBoxplot(
#'   data = ex.data, samples = ex.samples, score = score, bySamples = TRUE,
#'   main = "Some box plot"
#' )
makeBoxplot <- function(data, samples, score, bySamples = FALSE, col = "standard",
                        main = "", xlab = "Batch", ylab = "Beta value",
                        scoreCol = TRUE, log = FALSE) {
    logPar <- ""
    if(log){
        logPar <- "y"
    }
    
  if (bySamples == FALSE) {
    ## get batch numbers
    batches <- sort(unique(samples$batch_id))
    ## prepare data for the box plot
    boxplotData <- list()
    for (i in seq_len(length(batches)))
    {
      s <- as.character(samples[
        samples$batch_id == batches[i],
        "sample_id"
      ])
      boxplotData[[as.character(batches[i])]] <- as.vector(as.matrix(
        data[, s]
      ))
      remove(s)
    }
    annotation <- rep("", length(batches))
    par(las = 2)
    if (col == "standard") {
      coloring <- "yellow"
    }
    else {
      coloring <- col
    }

    ## make boxplot
    boxplot(boxplotData,
      col = coloring, main = main, xlab = xlab, ylab = ylab,
      names = annotation, cex.axis = 1, xaxt = "n", log = logPar
    )

    if (scoreCol == TRUE) {
      ## get scoring colors for the x-axis labels
      colors <- c()
      for (i in batches) {
        if (score$BEscore[score$batch == i] >= 0.1) {
          colors <- c(colors, "red")
        } else
        if (score$BEscore[score$batch == i] >= 0.02 &
          score$BEscore[score$batch == i] < 1) {
          colors <- c(colors, "orange")
        } else
        if (score$BEscore[score$batch == i] < 0.02) {
          colors <- c(colors, "darkgreen")
        }
      }
    }
    else {
      colors <- rep("black", length(batches))
    }

    ## add colored batch number to x-axis
    for (i in seq_len(length(batches))) {
      mtext(batches[i],
        side = 1, line = 1, at = i, las = 2, cex = 1,
        col = colors[i]
      )
    }
  }
  else {
    ## get batch numbers
    batches <- sort(unique(samples$batch_id))
    ## boxplot separated by sample
    ## sort samples by batch
    samplesNew <- samples
    samplesNew <- samplesNew[order(samplesNew$batch_id), ]
    ## take sample_ids
    barcodes <- samplesNew$sample_id

    ## prepare data
    boxplotData <- list()
    for (i in seq_len(length(barcodes)))
    {
      s <- as.character(samplesNew[
        samplesNew$sample_id == barcodes[i],
        "sample_id"
      ])
      boxplotData[[as.character(barcodes[i])]] <- as.vector(as.matrix(
        data[, s]
      ))
      remove(s)
    }

    annotation <- rep("", length(barcodes))
    par(las = 2)
    ## get colors for the boxes
    batchesForColoring <- sort(samples$batch_id)
    ## store number of samples per batch
    countings <- as.vector(table(batchesForColoring))
    if (col == "standard") {
      coloring <- c()
      colnum <- 1
      for (i in seq_len(length(batches))) {
        if (colnum == 1) {
          coloring <- c(coloring, rep("yellow", countings[i]))
          colnum <- 2
        } else
        if (colnum == 2) {
          coloring <- c(coloring, rep("red", countings[i]))
          colnum <- 3
        } else
        if (colnum == 3) {
          coloring <- c(coloring, rep("blue", countings[i]))
          colnum <- 4
        } else
        if (colnum == 4) {
          coloring <- c(coloring, rep(
            "green",
            countings[i]
          ))
          colnum <- 1
        }
      }
    }
    else {
      coloring <- col
    }

    ## make the boxplot
    boxplot(boxplotData,
      main = main, xlab = xlab, ylab = ylab,
      names = annotation, col = coloring, cex.axis = 0.8, xaxt = "n", log = logPar
    )

    ## get scoring colors for the x-axis labels
    if (scoreCol == TRUE) {
      colors <- c()
      for (i in batches) {
        if (score$BEscore[score$batch == i] >= 0.1) {
          colors <- c(colors, "red")
        } else
        if (score$BEscore[score$batch == i] >= 0.02 &
          score$BEscore[score$batch == i] < 1) {
          colors <- c(colors, "orange")
        } else
        if (score$BEscore[score$batch == i] < 0.02) {
          colors <- c(colors, "darkgreen")
        }
      }
    }
    else {
      colors <- rep("black", length(batches))
    }

    ## calculate positions to add text for every batch
    positionsText <- c()
    positionCounter <- 0
    for (i in seq_len(length(batches))) {
      if (countings[i] == 1) {
        positionsText <- c(positionsText, positionCounter + 1)
      } else
      if (countings[i] == 2) {
        positionsText <- c(positionsText, positionCounter + 1)
      } else
      if (countings[i] == 3) {
        positionsText <- c(positionsText, positionCounter + 2)
      } else
      ## nearly the mid position of the batch
      if (countings[i] >= 4) {
        positionsText <- c(positionsText, positionCounter +
          ceiling(countings[i] / 2))
      }
      positionCounter <- positionCounter + countings[i]
    }

    ## add colored batch number to x-axis
    batchCounter <- 1
    for (i in seq_len(positionCounter)) {
      if (batchCounter == length(batches) + 1) {
        break
      } else
      ## add number if i is on positionText of batch batchCounter
      if (i == positionsText[batchCounter]) {
        mtext(batches[batchCounter],
          side = 1, line = 1, at = i, las = 2,
          cex = 1, col = colors[batchCounter]
        )
        batchCounter <- batchCounter + 1
      } else
      ## add "" if not
      if (i != positionsText[batchCounter]) {
        mtext("", side = 1, line = 1, at = i, las = 2, cex = 1)
      }
    }
  }
}

#' calcBlockFrame
#'
#' @keywords internal
#'
#' @description  calculates every start and stop position for every block
#'
#' @return a data.frame containing the start and stop positions for each block
calcBlockFrame <-
  function(rowPositions,
             colPositions,
             rowBlockSize,
             colBlockSize) {
    ## construct data frame with all row-and column star-and stop positions
    blocks <- data.frame(
      number = numeric(),
      rowStartPos = numeric(),
      rowStopPos = numeric(),
      colStartPos = numeric(),
      colStopPos = numeric()
    )
    ## counters to choose the correct start - and stop positions within the
    ## loop
    rowCounter <- 1
    colCounter <- 1
    counter <- 1
    ## iteration over rows and columns to calculate the positions
    for (i in 0:(rowPositions[1] - 1)) {
      if (rowCounter < rowPositions[1] - 1) {
        currentRowStart <- i * rowBlockSize + 1
        currentRowStop <- i * rowBlockSize + rowBlockSize
        rowCounter <- rowCounter + 1
      } else {
        if (rowCounter == rowPositions[1] - 1) {
          currentRowStart <- rowPositions[2]
          currentRowStop <- rowPositions[4]
          rowCounter <- rowCounter + 1
        } else {
          if (rowCounter == rowPositions[1]) {
            currentRowStart <- rowPositions[3]
            currentRowStop <- rowPositions[5]
          }
        }
      }
      for (j in 0:(colPositions[1] - 1)) {
        if (colCounter < colPositions[1] - 1) {
          currentColStart <- j * colBlockSize + 1
          currentColStop <- j * colBlockSize + colBlockSize
          colCounter <- colCounter + 1
        } else {
          if (colCounter == colPositions[1] - 1) {
            currentColStart <- colPositions[2]
            currentColStop <- colPositions[4]
            colCounter <- colCounter + 1
          } else {
            if (colCounter == colPositions[1]) {
              currentColStart <- colPositions[3]
              currentColStop <- colPositions[5]
              colCounter <- 1
            }
          }
        }
        blocks <- rbind(
          blocks,
          data.frame(
            number = counter,
            rowStartPos = currentRowStart,
            rowStopPos = currentRowStop,
            colStartPos = currentColStart,
            colStopPos = currentColStop
          )
        )
        counter <- counter + 1
      }
    }
    return(blocks)
  }

testthat::context("Calculation of block positions")

testthat::test_that("Values that add up", {
  res1 <- calcBlockFrame(c(2, 1, 11, 10, 20), c(2, 1, 11, 10, 20), 10, 10)

  testthat::expect_equal(res1[, 1], c(1, 2, 3, 4))
  testthat::expect_equal(res1[, 2], c(1, 1, 11, 11))
  testthat::expect_equal(res1[, 3], c(10, 10, 20, 20))
  testthat::expect_equal(res1[, 4], c(1, 11, 1, 11))
  testthat::expect_equal(res1[, 5], c(10, 20, 10, 20))
})

testthat::test_that("Values that don't add up", {
  res1 <- calcBlockFrame(c(2, 1, 11, 10, 19), c(2, 1, 11, 10, 19), 11, 11)

  testthat::expect_equal(res1[, 1], c(1, 2, 3, 4))
  testthat::expect_equal(res1[, 2], c(1, 1, 11, 11))
  testthat::expect_equal(res1[, 3], c(10, 10, 19, 19))
  testthat::expect_equal(res1[, 4], c(1, 11, 1, 11))
  testthat::expect_equal(res1[, 5], c(10, 19, 10, 19))
})

testthat::context("Testing the imputation of missing values")


testthat::test_that("No NAs, serial", {
  library(seewave)
  bpparam <- SerialParam(log = TRUE, threshold = "INFO")
  data <- as.matrix(Hilbert(50))

  set.seed(1, kind = "Mersenne-Twister")
  res <- imputeMissingData(
    data = data, BPPARAM = bpparam,
    rowBlockSize = 10, colBlockSize = 10, outputFormat = ""
  )
  colnames(res) <- NULL
  testthat::expect_equal(object = as.vector(res), expected =  as.vector(data))
})

testthat::test_that("NAs, serial", {
  library(seewave)
  bpparam <- SerialParam(log = TRUE, threshold = "INFO")
  data <- as.matrix(Hilbert(50))

  data_missing <- data
  data_missing[1, 3] <- NA
  data_missing[9, 5] <- NA

  set.seed(1, kind = "Mersenne-Twister")
  res <- imputeMissingData(
    data = data_missing, BPPARAM = bpparam,
    rowBlockSize = 10, colBlockSize = 10, outputFormat = ""
  )
  colnames(res) <- NULL
  data[1, 3] <- 0.116
  data[9, 5] <- 0.020
  testthat::expect_equal(object = as.vector(res), expected =  as.vector(data), 
                         tolerance = .01)
})

---
title: "Detecting and correcting batch effects with BEclear"
author: 
    - name: Livia Rasp
      affiliation:
        Center for Bioinformatics, Saarland University, Saarbruecken, Germany
      email: Livia.Rasp@gmail.com
    - name: Markus Merl
package: BEclear
abstract: |
    We show in this tutorial how to use the BEclear package [@Akulenko2016] to detect and correct
    batch effects in methylation data. Even though BEclear was developed for
    the use on methylation data, it can also be used to find and correct batch 
    effects in other kinds of data.
    The central method of BEclear is based on Latent Factor Models [@Candes2009], which can in 
    theory be used on every matrix containing real numbers to predict missing values.
date: "`r Sys.Date()`"
output: 
    BiocStyle::html_document:
            toc_float: true
bibliography: "`r system.file('REFERENCES.bib', package = 'BEclear')`"
vignette: |
  %\VignetteIndexEntry{BEclear tutorial}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}  
---

```{r, echo = FALSE, message = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
library(pander)
panderOptions('knitr.auto.asis', TRUE)
panderOptions('plain.ascii', TRUE)
```

# Introduction

We guide you through the individual steps of the `r BiocStyle::Biocpkg("BEclear")`
package in their own chapters. They will follow in the logical order of an 
example of correcting some batch affected DNA methylation data. 
This article should only give a small tutorial,
more details about the individual methods can always be found in the help
sections of the `r BiocStyle::Biocpkg("BEclear")` package, e.g. through typing
`calcBatchEffects` in the R environment with the package loaded.
To work with the methods contained in the BEclear package, a matrix or
data.frame with genes as row-names and samples as column names as well as a
samples data.frame with the first column named "sample\_id" and the second
column named "batch\_id" is needed as input.

# Installation

`r BiocStyle::Biocpkg("BEclear")` is available on Bioconductor. To install it 
you can therefore use the `r BiocStyle::Biocpkg("BiocManager")`:

```{r, eval=FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE)) {
  install.packages("BiocManager")
}
BiocManager::install("BEclear")
```

Otherwise you can also install `r BiocStyle::Githubpkg("uds-helms/BEclear")`
from its Github repository by the following command:

```{r, eval=FALSE}
if (!requireNamespace("devtools", quietly = TRUE)) {
  install.packages("devtools")
}
devtools::install_github("uds-helms/BEclear")
```

We however recommend installing it through Bioconductor, as this takes care of 
installing the dependencies and furthermore you can refer to the release of 
Bioconductor, when using our package, which enables you to reproduce the exact 
conditions of your run.

During the compilation of the code, many parts of the software will be automatically
tested for correct execution and reproduction of expected results. This is implemented 
in form of unit tests with the help of the `r BiocStyle::CRANpkg("testthat")` package.

When done with the installation you can simply load the package by typing:

```{r}
library(BEclear)
```

# Data

The beta values stored in the ex.data matrix were obtained from level 3 BRCA
data from the TCGA portal [@TCGA]. Generally, beta values are calculated by
dividing the methylated signal by the sum of the unmethylated and methylated
signals from a DNA methylation microrarray. In the level 3 TCGA data, this
calculation has already been done. The sample data used here contains averaged
beta values of probes that belong to promoter regions of single genes. Another
possibility would be to use beta values of single probes, whereby the probe
names should then be used instead of the gene names as rownames of the matrix.

You can load our sample data via the following command:

```{r data}
data("BEclearData")
```

It contains one matrix with the beta values:

```{r}
knitr::kable(ex.data[1:10,1:5], caption = 'Some entries from the example data-set')
```

And one data.frame containing the assignment of samples to batches:

```{r}
knitr::kable(ex.samples[1:10,], caption = 'Some entries from the example sample annotation')
```

# Detection of batch effects

For the detection of batch effects we calculate the median difference between the 
beta values of a gene in a batch and the values of this gene in all other batches. 
Furthermore we use a non-parametric Kolmogorov-Smirnov test (`ks.test`) to  compare the
distribution of the beta value for this gene in the batch and the other batches.

If one gene in a batch has a p-value determined by the `ks.test` of less or equal
0.01 and a median difference of greater or equal 0.05 it is considered batch effected.

## Detection

For the calculation of the batch effects you just use the `calcBatchEffects` function.
It calculates both median difference and p-value. By default we correct the p-values
by the false discovery rate developed by @BH, but you can use all adjustment
methods covered by `p.adjust.methods`.

```{r detection, cache=TRUE}
batchEffect <- calcBatchEffects(
  data = ex.data, samples = ex.samples,
  adjusted = TRUE, method = "fdr"
)
mdifs <- batchEffect$med
pvals <- batchEffect$pval
```


## Summary

To see which genes in which batches are effected you use the `calcSummary` function 
as follows:

```{r summary, cache=TRUE}
summary <- calcSummary(medians = mdifs, pvalues = pvals)
knitr::kable(head(summary), caption = 'Summary over the batch affected gene-sample combination of the example data set')
```

## Scoring 

Furthermore you can calculate a batch score for a whole batch to determine the
severity how it is affected.

```{r score, cache=TRUE}
score <- calcScore(ex.data, ex.samples, summary, dir = getwd())
knitr::kable(score, caption = 'Batch scores of the example data-set')
```

# Imputation of missing values

For the imputation of missing values we use a slightly modified version of the
stochastic gradient descent method described by @Koren2009. 
In this section we will describe our implementation of this method and how to 
use it.


We assume that our complete data matrix \(D_{ij}\) can be described by the effects of
a matrix \(L_i\), which represents the effect of the features (genes in our case)
and a matrix \(R_j\) describing the effect of the samples in the following way:

\begin{equation}
D_{ij} = L_{i}^{T} \times R_{j} .
(\#eq:assumption)
\end{equation}

The method can either be run on the complete data set or the data set can be 
divided into blocks on which the method is applied.
This division into blocks allows for parallelisation of the method, which can be 
useful to speed up the process. We have found that a block-size of 60x60 works 
well[@Akulenko2016].

The error for each block is calculated in the following way:

\begin{equation}
  errorMatrix_{ij} = Block_{ij} - L_{i}^{T} \times R_{j} .
  (\#eq:errormatrix)
\end{equation}

We try to minimize the following loss function through a gradient descent:

\begin{equation}
  min_{L, R}  \sum_{ij \in K}(errorMatrix_{ij}^2) + \lambda \times
  (\left\lVert L_{i}\right\rVert_{F}^{2} + 
  \left\lVert R_{j}\right\rVert_{F}^{2} ).
  (\#eq:loss)
\end{equation}
Where \(K\) is the set of tuples \((i,j)\) for which the value is present. 
\(\lambda\) is the penalty coefficient, which controls how restrictive the 
selection of variables should be. The default of \(\lambda\) is 1.

Another coefficient \(\gamma\) controls the size of the step by which the 
two matrices \(L_i\) and \(R_j\) are modified. It is initialized 
by default with 0.01 and its value changes during the iterations (epochs).

For the first iteration the matrices \(L_i\) and \(R_j\) are filled with random values
generated by the `rnorm` function from the `r BiocStyle::Rpackage("stats")` 
package and the initial loss and error matrix are calculated.

Then for each iteration the following is done:
    
* \(L_i\) and \(R_j\) are modified proportional by \(\gamma\) through the following 
calculation:

    + \begin{equation}
      L_i = L_i + 2 \times \gamma \times  (errorMatrix_{ij} \times R_j - \lambda \times L_i).
      (\#eq:Lmod)
      \end{equation}

    + \begin{equation}
      R_j = R_j + 2 \times \gamma \times (errorMatrix_{ij} \times L_i - \lambda \times R_j).
      (\#eq:Rmod)
      \end{equation}

* Then the new error matrix and loss are calculated.
* If the old loss is smaller than the new one: 
    + \(\gamma = \gamma \div 2.\)
* Else:
    + \(\gamma = \gamma \times 1.05.\)
    
The \(L_i\) and \(R_j\) matrices at the end of the last iteration are then used to 
impute the missing data. The default number of iterations is 50.

## Usage

First you have to set the found batch effect values to NAs. You can do this
by using the `clearBEgenes` function:

```{r clearBE, cache=TRUE}
cleared.data <- clearBEgenes(ex.data, ex.samples, summary)
```
In case you're using `r BiocStyle::Biocpkg("BEclear")` not for correcting batch
effects, but just for the data imputation, you would have to set the values you
want to impute to NA, if they not already are. 

For the data imputation you use the `imputeMissingData` function:

```{r imputation, cache=TRUE}
library(ids)
corrected.data <- imputeMissingData(cleared.data,
  rowBlockSize = 60,
  colBlockSize = 60, epochs = 50,
  outputFormat = "", dir = getwd()
)
```

If you set rowBlockSize and colBlockSize to 0 the matrix will not be divided into 
block and the gradient descent will be applied to the matrix as a whole.

## Replacing values outside the boundaries

Note that sometimes during the prediction, it can happen that values beyond the
boundaries of beta values are returned, that means values smaller than zero or
greater than one. `findWrongValues` simply returns a list of these values,
together with the position in the output matrix, `replaceOutsideValues` corrects
these by simply setting the wrong values to zero or one, respectively. Note that 
these methods are especially designed for the prediction of beta values from 
DNA methylation data, which only take on values between 0 and 1. 


```{r replace, cache=TRUE}
corrected.data.valid<-replaceOutsideValues(corrected.data)
```

In this case there were no values to be replaced.


# Overall correction

Besides the individual methods BEclear also offers an overall method, which 
executes all the described previous steps in one call. It also applies some 
preprocessing to your data set if necessary.

```{r correction, cache=TRUE}
result <- correctBatchEffect(data = ex.data, samples = ex.samples)
```

Returned is a list containing all results from the executed functions.

# Parallelization

For parallelization we use the `r BiocStyle::Biocpkg("BiocParellel")` package.
However by default all methods are executed in serial mode.
The methods `CalcBatchEffect`, `imputeMissingData` and `correctBatchEffect` 
support parallelization through there parameter `BPPARAM`, which takes a `BiocParallel::BiocParallelParam` class as an argument. 

Type the following to get an overview over the supported evaluation environments:

```{r}
?BiocParallel::BiocParallelParam
```
# Plotting

Additionally `r BiocStyle::Biocpkg("BEclear")` also includes a method for 
plotting the batch effects.
Let us now use the `makeBoxplot` to compare the distributions of the values
in the different samples before and after the batch effect correction:

```{r boxplot1, fig.wide = TRUE, fig.cap = "Distribution of the example beta values grouped by sample"}
makeBoxplot(ex.data, ex.samples, score,
  bySamples = TRUE,
  col = "standard", main = "Example data", xlab = "Batch",
  ylab = "Beta value", scoreCol = TRUE)
```

```{r boxplot2, fig.wide = TRUE, fig.cap = "Distribution of the corrected beta values grouped by sample"}
makeBoxplot(corrected.data, ex.samples, score,
  bySamples = TRUE,
  col = "standard", main = "Corrected example data",
  xlab = "Batch", ylab = "Beta value", scoreCol = FALSE)
```

# Session info {.unnumbered}

Here is the output of `sessionInfo()` on the system on which this document 
was compiled running pandoc `r rmarkdown::pandoc_version()`:

```{r sessionInfo, echo=FALSE}
pander(sessionInfo(), compact=TRUE)
```

# References {.unnumbered}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BEclear.R
\docType{data}
\name{ex.corrected.data}
\alias{ex.corrected.data}
\title{Example matrix of corrected data for the BEclear-package}
\format{
A matrix containing already corrected beta values of some samples from
the breast invasive carcinoma TCGA methylation data. The colnames denote
samples, rownames denote gene names.
}
\usage{
data(BEclearCorrected)
}
\description{
Example matrix containing a already batch effect corrected sample
matrix of beta values from breast invasive carcinoma TCGA methylation
data.[1] The matrix contains a small amount of predicted beta values outside of
the boundaries to show the operating principles of some of the methods from 
the BEclear package.
}
\references{
\insertRef{TCGA}{BEclear}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BEclear.R
\docType{data}
\name{BEclear example methylation data}
\alias{BEclear example methylation data}
\alias{ex.data}
\title{Example data set for the BEclear-package}
\format{
An example data matrix that is filled with beta values originally
stemming from breast cancer data from the TCGA portal [1], colnames are
sample ids, rownames are gene names. Generally, beta values are calculated by
dividing the methylated signal by the sum of the unmethylated and methylated
signals from a DNA methylation microrarray. The sample data used here
contains averaged beta values of probes that belong to promoter regions of
single genes. Another possibility would be to use beta values of single
probes, whereby the probe names should then be used instead of the gene names
as rownames of the matrix.
}
\usage{
data(BEclearData)
}
\description{
Example data set for the BEclear-package
}
\references{
\insertRef{TCGA}{BEclear}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calcSummary.R
\name{calcSummary}
\alias{calcSummary}
\title{Summarize median comparison and p-value calculation results}
\usage{
calcSummary(medians, pvalues, mediansTreshold, pvaluesTreshold)
}
\arguments{
\item{medians}{a matrix containing median difference values calculated by
the \code{\link{calcBatchEffects}} function. For further details look at the
documentation of this function.}

\item{pvalues}{a matrix containing p-values calculated by the
\code{\link{calcBatchEffects}} function. For further details look at the
documentation of this function.}

\item{mediansTreshold}{the threshold above or equal median values are regarded 
as batch effected, when the criteria for p-values is also met.}

\item{pvaluesTreshold}{the threshold below or equal p-values are regarded as 
batch effected, when the criteria for medians is also met.}
}
\value{
Null if there are no batch effects detected, else
a \code{\link[data.table]{data.table}} with the columns "gene" containing the gene name,
"batch" containing the batch number from which the gene was found, "median"
and "p-value" containing the calculated median difference values and the
p-values, respectively.
}
\description{
Summarizes the results of the \code{\link{calcBatchEffects}} 
function
}
\details{
calcSummary

All genes with a median comparison value >= 0.05 and a p-value
of <= 0.01 are summarized into a data.frame. These genes are assumed to
contain a batch effect
}
\examples{
## Shortly running example. For a more realistic example that takes
## some more time, run the same procedure with the full BEclearData
## dataset.

## Whole procedure that has to be done to use this function.
data(BEclearData)
ex.data <- ex.data[31:90, 7:26]
ex.samples <- ex.samples[7:26, ]

## Calculate the batch effects
batchEffects <- calcBatchEffects(data = ex.data, samples = ex.samples,
adjusted = TRUE, method = "fdr")
med <- batchEffects$med
pvals <- batchEffects$pval

## Summarize p-values and median differences for batch affected genes
sum <- calcSummary(medians = med, pvalues = pvals)
}
\seealso{
\code{\link{calcBatchEffects}}

\code{\link{correctBatchEffect}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BEclear.R
\docType{package}
\name{BEclear-package}
\alias{BEclear-package}
\alias{_PACKAGE}
\alias{BEclearCorrected}
\title{Correction of batch effects in DNA methylation data}
\description{
Provides some functions to detect and correct for batch effects
in DNA methylation data. The core function \code{\link{correctBatchEffect}} is based on
Latent Factor Models and can also be used to predict missing values in any
other matrix containing real numbers.
}
\details{
BEclear-package

\code{\link{correctBatchEffect}}:
The function combines most functions of the \code{\link{BEclear-package}} to
one. This function performs the whole process of searching for batch effects
and automatically correct them for a matrix of beta values stemming from DNA
methylation data.\cr
\code{\link{correctBatchEffect}}:
This function predicts the missing entries of an input matrix (NA values)
through the use of a Latent Factor Model.\cr
\code{\link{calcBatchEffects}}:
Compares the median value of all beta values belonging to one batch with the
median value of all beta values belonging to all other batches. Returns a
matrix containing this median difference value for every gene in every batch,
columns define the batch numbers, rows the gene names.\cr
And compares the distribution of all beta values corresponding to one batch 
with the distribution of all beta values corresponding to all other batches and
returns a p-value which defines if the distributions are the same or not.\cr
\code{\link{calcSummary}}:
Summarizes the results of the \code{\link{calcBatchEffects}} function\cr
\code{\link{calcScore}}:
Returns a table with the number of found genes with found p-values less or
equal to 0.01 and median values greater or equal to 0.05. A score is
calculated depending on the number of found genes as well as the magnitude of
the median difference values, this score is divided by the overall number of
genes in the data and returned as "BEscore". See the methods details for
further information and details about the score calculation.\cr
\code{\link{makeBoxplot}}:
A simple \code{\link{boxplot}} is done with boxes either separated by batches
or by samples and describe the five number summary of all beta values
corresponding to a batch or a sample, respectively. The batch_ids are shown on
the x-axis with a coloring corresponding to the BEscore.\cr
\code{\link{clearBEgenes}}:
A function that simply sets all values to NA which were previously found by
median value comparison and p-value calculation and are stored in a summary.
The summary defines which values in the data matrix are set to NA.\cr
\code{\link{countValuesToPredict}}:
Simple function that counts all values in a matrix which are NA\cr
\code{\link{findOutsideValues}}:
A method which lists values below 0 or beyond 1 contained in the input matrix.
These entries are stored in a data.frame together with the corresponding
row and column position of the matrix.\cr
\code{\link{replaceOutsideValues}}:
A method which replaces values below 0 or beyond 1 contained in the input
matrix. These entries outside the boundaries are replaced by 0 or 1, respectively.
}
\examples{
data(BEclearData)
## Calculate the batch effects
batchEffects <- calcBatchEffects(data = ex.data, samples = ex.samples,
adjusted = TRUE, method = "fdr")
med <- batchEffects$med
pvals <- batchEffects$pval

## Summarize p-values and median differences for batch affected genes
sum <- calcSummary(medians = med, pvalues = pvals)

## Calculates the score table
score.table <- calcScore(data = ex.data, samples = ex.samples, summary = sum)

## Simple boxplot for the example data separated by batch
makeBoxplot(
  data = ex.data, samples = ex.samples, score = score.table,
  bySamples = FALSE, main = "Some box plot"
)

## Simple boxplot for the example data separated by samples
makeBoxplot(
  data = ex.data, samples = ex.samples, score = score.table,
  bySamples = TRUE, main = "Some box plot"
)

## Sets assumed batch affected entries to NA
cleared <- clearBEgenes(data = ex.data, samples = ex.samples, summary = sum)
## Counts and stores number of entries to predict
numberOfEntries <- countValuesToPredict(data = cleared)
\dontrun{
## Predicts the missing entries
predicted <- imputeMissingData(data = cleared)

## Find predicted entries outside the boundaries
outsideEntries <- findOutsideValues(data = predicted)

## Replace predicted entries outside the boundaries
corrected <- replaceOutsideValues(data = predicted)
}

}
\references{
\insertRef{Akulenko2016}{BEclear}
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/uds-helms/BEclear}
  \item Report bugs at \url{https://github.com/uds-helms/BEclear/issues}
}

}
\author{
Ruslan Akulenko, Markus Merl, Livia Rasp
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/combineBlocks.R
\name{combineBlocks}
\alias{combineBlocks}
\title{combineBlocks}
\usage{
combineBlocks(blockFrame, rowPos, colPos, dir)
}
\value{
the combined matrix consisting of the individual blocks
}
\description{
load and combine blocks
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calcScore.R
\name{calcScore}
\alias{calcScore}
\title{calculate batch effect score}
\usage{
calcScore(data, samples, summary, saveAsFile=FALSE, dir=getwd())
}
\arguments{
\item{data}{any matrix filled with beta values, column names have to be
sample_ids corresponding to the ids listed in "samples", row names have to
be gene names.}

\item{samples}{data frame with two columns, the first column has to contain
the sample numbers, the second column has to contain the corresponding batch
number. Colnames have to be named as "sample_id" and "batch_id".}

\item{summary}{a summary \code{\link[data.table]{data.table}} containing the 
columns "gene", "batch", median" and "p-value" and consists of all genes which 
were found in the median and p-value calculations, see 
\code{\link{calcSummary}} function for more details.}

\item{saveAsFile}{determining if the data.frame should also be saved as a file}

\item{dir}{set the path to a directory the returned data.frame should be
stored. The current working directory is defined as default parameter.}
}
\value{
A data.frame is returned containing the number of found genes assumed
to be batch affected separated by batch and a BEscore for every batch. Furthermore
there's a column dixonPval giving you a p-value regarding each BEscore according 
to a Dixon test.
The data.frame is also stored in the specified directory as .RData file, if 
saveAsFile is \code{TRUE}.
}
\description{
Returns a table with the number of found genes with found
p-values less or equal to 0.01 and median values greater or equal to 0.05.
A score is calculated depending on the number of found genes as well as the
magnitude of the median difference values, this score is divided by the
overall number of genes in the data and returned as "BEscore". See details
for further information and details about the score calculation. The
returned data.frame is also stored in the specified directory as .RData file.
}
\details{
calcScore

The returned data frame contains one column for the batch numbers,
11 columns containing the number of genes found in a certain range of the
median difference value and a column with the calculated BEscore. These
found genes are assumed to be batch affected due to their difference in
median values and their different distribution of the beta values. The higher
the found number of genes and the more extreme the median difference is, the
more severe is the assumed batch effect supposed to be. We suggest that there
is no need for a batch effect correction if the BEscore for a batch is less
than 0.02. BEscores between 0.02 and 0.1 are lying in a "gray area" for which
we assume a not severe batch effect, and values beyond 0.1 certainly describe
a batch effect and should definitely be corrected.\cr
The 11 columns containing the numbers of found genes count the median
difference values which are ranging from >= 0.05 to < 0.1 ; >= 0.1 to < 0.2;
>= 0.2 to < 0.3 and so on up to a limit of 1.\cr
The BEscore is calculated by the sum of the weighted number of genes divided
by the number of genes. Weightings are calculated by multiplication of the
number of found genes between 0.05 and 0.1 by 1, between 0.1 and 0.2 by 2,
between 0.2 and 0.3 by 4, between 0.3 and 0.4 by 6 and so on.
}
\examples{
## Shortly running example. For a more realistic example that takes
## some more time, run the same procedure with the full BEclearData
## dataset.

## Whole procedure that has to be done to use this function.
data(BEclearData)
ex.data <- ex.data[31:90, 7:26]
ex.samples <- ex.samples[7:26, ]
## Calculate the batch effects
batchEffects <- calcBatchEffects(data = ex.data, samples = ex.samples,
adjusted = TRUE, method = "fdr")
med <- batchEffects$med
pvals <- batchEffects$pval

# Summarize p-values and median differences for batch affected genes
sum <- calcSummary(medians = med, pvalues = pvals)

# Calculates the score table
score.table <- calcScore(data = ex.data, samples = ex.samples, summary = sum)
}
\references{
\insertRef{Dixon1950}{BEclear}

\insertRef{Dixon1951}{BEclear}

\insertRef{Rorabacher1991}{BEclear}
}
\seealso{
\code{\link{calcBatchEffects}}

\code{\link{calcSummary}}

\code{\link{correctBatchEffect}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/preprocessBEclear.R
\name{preprocessBEclear}
\alias{preprocessBEclear}
\title{preprocessBEclear}
\usage{
preprocessBEclear(data, samples)
}
\arguments{
\item{data}{any matrix filled with beta values, column names have to be
sample_ids corresponding to the ids listed in "samples", row names have to
be gene names.}

\item{samples}{data frame with two columns, the first column has to contain
the sample numbers, the second column has to contain the corresponding batch
number. Colnames have to be named as "sample_id" and "batch_id".}
}
\value{
a list containing the processed data and samples and a 
\code{\link[data.table]{data.table}} containing mappings from the original
sample names to the new ones. If sample names weren't changed this third
object is \code{NULL}
}
\description{
this methods does some preprocessing steps for the later methods
like removing rows containing only missing values
}
\details{
Here we describe the preprocessing steps in the order they are executed:
\itemize{
\item{Values below 0 or above 1 are set to \code{NA}, as the other methods 
expect methylation beta values}
\item{columns that only contain \code{NA}s are removed}
\item{rows that only contain \code{NA}s are removed}
\item{samples that are present in the data, but are not annoted in the samples 
are removed. If this is the case with your data-set, please check those samples.}
\item{samples that are annoted but not in the data matrix are removed}
\item{if there are duplicate sample names in the data matrix, all sample names
get replaced through a new unique ID. In this case a \code{\link[data.table]{data.table}}
containing the mapping is returned as well}
}
}
\examples{
data(BEclearData)
res <- preprocessBEclear(ex.data, ex.samples)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/findOutsideValues.R
\name{findOutsideValues}
\alias{findOutsideValues}
\title{Find DNA methylation values out of the boundaries}
\usage{
findOutsideValues(data)
}
\arguments{
\item{data}{any matrix filled with values that normally should be bounded
between 0 and 1.}
}
\value{
A data frame containing the columns "level", "row", "col" and "value"
defining if the value is below 0 or beyond 1 (level = 0 or level = 1),
the row position and the column position in the input matrix and the
value itself, respectively.
}
\description{
A method which lists values below 0 or beyond 1 contained in the
input matrix. These entries are stored in a data.frame together with the
corresponding row and column position of the matrix. Note that this method is
especially designed for DNA methylation data.
}
\details{
findOutsideValues

Note that this method is especially designed to run after the batch
effect correction of DNA methylation data, e.g. with the
\code{\link{correctBatchEffect}} method. It can happen, that the predicted values are
lying slightly below the lower bound of 0 or beyond the upper bound of 1.
This method finds these inaccurately predicted entries. Another method called
\code{\link{replaceOutsideValues}} replaces these values either by 0 or 1,
respectively.
}
\examples{
data(BEclearCorrected)
# Find predicted values outside of the boundaries
outsideEntries <- findOutsideValues(data = ex.corrected.data)
}
\seealso{
\code{\link{replaceOutsideValues}}

\code{\link{correctBatchEffect}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calcBlockFrame.R
\name{calcBlockFrame}
\alias{calcBlockFrame}
\title{calcBlockFrame}
\usage{
calcBlockFrame(rowPositions, colPositions, rowBlockSize, colBlockSize)
}
\value{
a data.frame containing the start and stop positions for each block
}
\description{
calculates every start and stop position for every block
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calcBatchEffectsForBatch.R
\name{calcBatchEffectsForBatch}
\alias{calcBatchEffectsForBatch}
\title{calcBatchEffectsForBatch}
\usage{
calcBatchEffectsForBatch(batch, samples, data, BPPARAM = SerialParam())
}
\value{
the medians p-values for genes in a batch
}
\description{
function to calculate batch effects for every gene in a batch
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/makeBoxplot.R
\name{makeBoxplot}
\alias{makeBoxplot}
\title{produce simple predefined boxplot for methylation data}
\usage{
makeBoxplot(data, samples, score, bySamples=FALSE, col="standard",
main="", xlab="Batch", ylab="Beta value", scoreCol=TRUE, log = FALSE)
}
\arguments{
\item{data}{any matrix filled with beta values, column names have to be
sample_ids corresponding to the ids listed in "samples", row names have to
be gene names.}

\item{samples}{data frame with two columns, the first column has to contain
the sample numbers, the second column has to contain the corresponding batch
number. Colnames have to be named as "sample_id" and "batch_id".}

\item{score}{data frame produced by the \code{\link{calcScore}} function.
Contains the number of presumably batch affected genes and a BEscore which
is needed for the coloring of the batch_ids.}

\item{bySamples}{should the boxes be separated by samples or not. If not,
boxes are separated by the batch_ids.}

\item{col}{colors for the boxes, refers to the standard \code{\link{boxplot}}
R-function. If it is set to "standard", boxes are colored batch-wise (if
separated by samples) or the standard color "yellow" is used (if separated
by batches).}

\item{main}{main title for the box plot. Default is an empty string.}

\item{xlab}{label for the x-axis of the box plot. Default is "Batch".}

\item{ylab}{label for the y-axis of the box plot. Default is "Beta value".}

\item{scoreCol}{should the batch_ids on the a-axis be colored according to
the BEscore or not? If not, black is used as color for all batch_ids.}

\item{log}{TRUE, if the y-axis should be on a logarithmic scale.}
}
\value{
Returns a boxplot on the graphic device with the features explained
above.
}
\description{
A simple \code{\link{boxplot}} is done with boxes either
separated by batches or by samples and describe the five number summary of
all beta values corresponding to a batch or a sample, respectively. The
batch_ids are shown on the x-axis with a coloring corresponding to the
BEscore.
}
\details{
makeBoxplot

The color code for the batch_ids on the x-axis provides a simple
"traffic light" the user can use to decide if he wants to correct for an
assumed batch effect or not. Green means no batch effect, yellow a possibly
existing not severe batch effect and red stands for an obviously existing
batch effect that should be corrected. The traffic light colors are set
according to the BEscore from the \code{\link{calcScore}} function, values
from 0 to 0.02 are colored in green, from 0.02 to 0.1 in yellow and values
over 0.1 are colored in red.
}
\examples{
## Shortly running example. For a more realistic example that takes
## some more time, run the same procedure with the full BEclearData
## dataset.

## Whole procedure that has to be done to use this function.
data(BEclearData)
ex.data <- ex.data[31:90, 7:26]
ex.samples <- ex.samples[7:26, ]

## Prepare the data for the box plots
## Calculate the batch effects
batchEffects <- calcBatchEffects(data = ex.data, samples = ex.samples,
adjusted = TRUE, method = "fdr")
meds <- batchEffects$med
pvals <- batchEffects$pval

## Summarize p-values and median differences for batch affected genes
sum <- calcSummary(medians = meds, pvalues = pvals)

# Calculate the BEscore for the batch_id colorings of the x-axis
score <- calcScore(data = ex.data, samples = ex.samples, summary = sum)

## Simple boxplot for the example data separated by batch
makeBoxplot(
  data = ex.data, samples = ex.samples, score = score, bySamples = FALSE,
  main = "Some box plot"
)

## Simple boxplot for the example data separated by samples
makeBoxplot(
  data = ex.data, samples = ex.samples, score = score, bySamples = TRUE,
  main = "Some box plot"
)
}
\seealso{
\code{\link{calcScore}}

\code{\link{boxplot}}

\code{\link{correctBatchEffect}}
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loss.R
\name{loss}
\alias{loss}
\title{loss}
\usage{
loss(L, R, lambda, D)
}
\arguments{
\item{L}{a matrix describing the effects of the features}

\item{R}{a matrix describing the effects of the samples}

\item{lambda}{constant that controls the extent of regularization during the
gradient descent}

\item{D}{a matrix containing the measured values}
}
\value{
a list containing the loss calculated and the error matrix
}
\description{
computation of the loss of factorization LR
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/countValuesToPredict.R
\name{countValuesToPredict}
\alias{countValuesToPredict}
\title{Count NA entries in a matrix}
\usage{
countValuesToPredict(data)
}
\arguments{
\item{data}{any kind of matrix}
}
\value{
Returns a data frame with the number of NA entries per column. Since
the function is mainly written for the usage in batch effect correction of
DNA methylation data, the column names of the data frame are set to "sample"
and "num_pred_values". Nevertheless, the function can be used with any other
matrix containing anything but beta values.
}
\description{
Simple function that counts all values in a matrix which are NA
}
\details{
countValuesToPredict
}
\examples{
## Shortly running example. For a more realistic example that takes
## some more time, run the same procedure with the full BEclearData
## dataset

## Whole procedure that has to be done to use this function.
data(BEclearData)
ex.data <- ex.data[31:90, 7:26]
ex.samples <- ex.samples[7:26, ]

## Calculate the batch effects
batchEffects <- calcBatchEffects(data = ex.data, samples = ex.samples,
adjusted = TRUE, method = "fdr")
meds <- batchEffects$med
pvals <- batchEffects$pval

## Summarize p-values and median differences for batch affected genes
sum <- calcSummary(medians = meds, pvalues = pvals)
clearedMatrix <- clearBEgenes(data = ex.data, samples = ex.samples, summary = sum)
numberOfEntries <- countValuesToPredict(data = clearedMatrix)
}
\seealso{
\code{\link{clearBEgenes}}

\code{\link{correctBatchEffect}}

\code{\link{correctBatchEffect}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{localLoss}
\alias{localLoss}
\title{Calculating the Local Loss}
\usage{
localLoss(L, R, is, js, error_matrix)
}
\value{
a list containing two matrices containing the local loss
}
\description{
Calculates the gradient dR and dL for the current epoch.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calcPositions.R
\name{calcPositions}
\alias{calcPositions}
\title{calcPositions}
\usage{
calcPositions(num, blockSize)
}
\value{
returns vector with number of blocks, start - and stop position of
last 2 blocks
}
\description{
calcPositions
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runGradientDescent.R
\name{runGradientDescent}
\alias{runGradientDescent}
\title{runGradientDescent}
\usage{
runGradientDescent(L, R, lambda, epochs, gamma = 0.01, blockNr, is, js, D, r)
}
\value{
a list containing two matrices generated by the gradient descent
}
\description{
Runner for gradient descent (or stochastic gradient descent) for
the specified number of epoch
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/imputeMissingData.R
\name{imputeMissingData}
\alias{imputeMissingData}
\title{Matrix prediction using a Latent Factor Model}
\usage{
imputeMissingData(data, rowBlockSize=60,  colBlockSize=60, epochs=50,
lambda = 1, gamma = 0.01, r = 10, outputFormat="", dir = tempdir(),
BPPARAM=SerialParam())
}
\arguments{
\item{data}{any matrix filled e.g. with beta values. The missing entries you
want to predict have to be set to NA}

\item{rowBlockSize}{the number of rows that is used in a block if the
function is run in parallel mode and/or not on the whole matrix. Set this and
the "colBlockSize" parameter to 0 if you want to run the function on the
whole input matrix. We suggest to use a block size of 60 but you can also use
any other block size, but the size has to be bigger than the number of
samples in the biggest batch. Look at the details section for more
information about this feature.}

\item{colBlockSize}{the number of columns that is used in a block if the
function is run in parallel mode and/or not on the whole matrix. Set this,
and the "rowBlockSize" parameter to 0 if you want to run the function on the
whole input matrix. We suggest to use a block size of 60 but you can also use
any other block size, but the size has to be bigger than the number of
samples in the biggest batch. Look at the details section for more
information about this feature.}

\item{epochs}{the number of iterations used in the gradient descent algorithm
to predict the missing entries in the data matrix.}

\item{lambda}{constant that controls the extent of regularization during the
gradient descent}

\item{gamma}{constant that controls the extent of the shift of parameters
during the gradient descent}

\item{r}{length of the second dimension of variable matrices R and L}

\item{outputFormat}{you can choose if the finally returned data matrix should
be saved as an .RData file or as a tab-delimited .txt file in the specified
directory. Allowed values are "RData" and "txt".}

\item{dir}{set the path to a directory the predicted matrix should be
stored. The current working directory is defined as default parameter.}

\item{BPPARAM}{An instance of the
\code{\link[BiocParallel]{BiocParallelParam-class}} that determines how to
parallelisation of the functions will be evaluated.}
}
\value{
Returns a data matrix with the same dimensions as well as same row
and column names as the input matrix. According to the "outputFormat"
parameter, either a .RData file containing only the returned matrix or a
tab-delimited .txt file containing the content of the returned matrix is
saved in the specified directory.
}
\description{
This function predicts the missing entries of an input matrix
(NA values) through the use of a Latent Factor Model. You can run the
function also in parallel mode and split up the matrix to a certain number of
smaller matrices to speed up the prediction process. If you set the
rowBlockSize and colBlockSize both to 0, the function is running on the whole
matrix. Take a look at the details section for some deeper information about
this. The default parameters are chosen with the intention to make an
accurate prediction within an affordable time interval.
}
\details{
imputeMissingData

The method used to predict the missing entries in the matrix is
called "latent factor model". In the following sections, the method itself is
described as well as the correct usage of the parameters. The parameters are
described in the same order as they appear in the usage section.\cr
The method originally stems from recommender systems where the goal is to
predict user ratings of products. It is based on matrix factorization and uses
a discrete gradient descent (GDE) algorithm that stepwise predicts two
matrices L and R with matching dimensions to the input matrix. These two
matrices are initialized with random numbers and stepwise adjusted towards
the values of the input matrix through the GDE algorithm. After every
adjustment step, the global loss is calculated and the parameters used for
the adjustment are possibly also adjusted so that the global loss is getting
minimized and the prediction is getting accurate. After a predefined number
of steps (called epochs) are executed by the GDE algorithm, the predicted
matrix is calculated by matrix multiplication of L and R. Finally, all
missing values in the input matrix are replaced with the values from the
predicted matrix and the already known values from the input matrix are
maintained. The completed input matrix is then returned at the end.\cr
Description of the parameters:
\itemize{
\item data: simply the input matrix with missing values set to NA
\item rowBlockSize and colBlockSize: Here you can define the dimensions of
the smaller matrices, the input matrix is divided into if the function is
working in parallel mode. For details about these so called blocks, see the
section "About the blocks" below.
\item epochs: Defines the number of steps the gradient descent algorithm
performs until the prediction ends. Note that the higher this number is, the
more precisely is the prediction and the more time is needed to perform the
prediction. If the step size is too small, the prediction would not be very
good. We suggest to use a step size of 50 since we did not get better
predictions if we took higher step sizes during our testing process.
}
About the blocks:
You have the possibility to change the size of the blocks in which the input
matrix can be divided. if you choose e.g. the rowBlockSize = 50 and the
colBlockSize = 60 your matrix will be cut into smaller matrices of the size
approximately 50x60. Note that this splitting algorithm works with every
possible matrix size! If both size parameters do not fit to the dimensions of
the input matrix, the remaining rows and columns of the input matrix are
distributed over some blocks, so that the block sizes are roughly of the same
size. All blocks are saved at the specified directory after the processing
of a block has been done within an RData file. These RData files are
continuously numbered and contain the row and column start and stop
positions in their name. Next, these blocks are assembled into the returned
matrix and this matrix is saved in the specified directory. Finally, single
blocks are deleted. To see how this is done, simply run the example at the
end of this documentation. We suggest to use the block size of 60 (default)
but you can also use any other block size, as far as it is bigger than the
number of samples in the biggest batch. This avoids having an entire row of
NA values in a block which leads to a crash of the imputeMissingData method.
In order  to process the complete matrix without dividing into blocks,
specify rowBlockSize = 0 and colBlockSize = 0. But if the input matrix is
large (more than 200x200), it is not recommended due to exponential increase
of computation time required.\cr
Note that the size of the blocks affect the prediction accuracy. In case of
very small blocks, the information obtained from neighbor entries is not
sufficient. Thus, the larger the size of the block is, the more accurately
those entries are predicted. Default size 60 is enough to have accurate
prediction in a reasonable amount of time.
}
\examples{
## Shortly running example. For a more realistic example that takes
## some more time, run the same procedure with the full BEclearData
## dataset.

## Whole procedure that has to be done to use this function.
data(BEclearData)
ex.data <- ex.data[31:90, 7:26]
ex.samples <- ex.samples[7:26, ]

## Calculate the batch effects
batchEffects <- calcBatchEffects(data = ex.data, samples = ex.samples,
adjusted = TRUE, method = "fdr")
meds <- batchEffects$med
pvals <- batchEffects$pval

## Summarize p-values and median differences for batch affected genes
sum <- calcSummary(medians = meds, pvalues = pvals)

## Set entries defined by the summary to NA
clearedMatrix <- clearBEgenes(data = ex.data, samples = ex.samples, summary = sum)

# Predict the missing entries with standard values, row- and block sizes are
# just set to 10 to get a short runtime. To use these parameters, either use
# the default values or please note the description in the details section
# above
predicted <- imputeMissingData(
  data = clearedMatrix, rowBlockSize = 10,
  colBlockSize = 10
)
}
\references{
\insertRef{Akulenko2016}{BEclear}

\insertRef{Koren2009}{BEclear}

\insertRef{Candes2009}{BEclear}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BEclear.R
\docType{data}
\name{BEclear example sample data}
\alias{BEclear example sample data}
\alias{ex.samples}
\title{Example data set for the BEclear-package}
\format{
An example data frame containing a column for the sample id and a
column for the corresponding batch id, stemming from breast cancer data
from the TCGA portal [1]
}
\usage{
data(BEclearData)
}
\description{
Example data set for the BEclear-package
}
\references{
\insertRef{TCGA}{BEclear}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/replaceOutsideValues.R
\name{replaceOutsideValues}
\alias{replaceOutsideValues}
\title{Replace DNA methylation values out of the boundaries}
\usage{
replaceOutsideValues(data)
}
\arguments{
\item{data}{any matrix filled with values that normally should be bounded
between 0 and 1.}
}
\value{
Returns the input matrix with every value previously below 0 changed
to 0 and every value previously beyond 1 changed to 1.
}
\description{
A method which replaces values below 0 or beyond 1 contained in
the input matrix. These wrong entries are replaced by 0 or 1, respectively.
Note that this method is especially designed for DNA methylation data.
}
\details{
replaceOutsideValues

Note that this method is especially designed to run after the batch
effect correction of DNA methylation data, e.g. with the
\code{\link{imputeMissingData}} method. It can happen, that the predicted
values are lying slightly below the lower bound of 0 or beyond the upper
bound of 1. This method finds these inaccurately predicted entries. Another
method called \code{\link{replaceOutsideValues}} replaces these values either
by 0 or 1, respectively. Another method called \code{\link{findOutsideValues}}
returns a list of existing wrong values and can be run before the
replacement.
}
\examples{
data(BEclearCorrected)
# Replace wrongly predicted values
corrected <- replaceOutsideValues(data = ex.corrected.data)
}
\seealso{
\code{\link{findOutsideValues}}

\code{\link{correctBatchEffect}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clearBEgenes.R
\name{clearBEgenes}
\alias{clearBEgenes}
\title{Prepare a data matrix for the BEclear function}
\usage{
clearBEgenes(data, samples, summary)
}
\arguments{
\item{data}{any matrix filled with beta values, column names have to be
sample_ids corresponding to the ids listed in "samples", row names have to be
gene names.}

\item{samples}{data frame with two columns, the first column has to contain
the sample numbers, the second column has to contain the corresponding batch
number. Colnames have to be named as "sample_id" and "batch_id".}

\item{summary}{a summary data.frame containing the columns "gene", "batch",
"median" and "p-value" and consists of all genes which were found in the
median and p-value calculations, see \code{\link{calcSummary}} function for
more details.}
}
\value{
A data matrix with the same dimensions as well as the same column
and row names as the input data matrix is returned, all entries which are
defined in the summary are now set to NA.
}
\description{
A function that simply sets all values to NA which were
previously found by median value comparison and p-value calculation and are
stored in a summary. The summary defines which values in the data matrix are
set to NA.
}
\details{
clearBEgenes

All entries belonging to genes stated in the summary are set to NA
for the corresponding batches in the data matrix. Please look at the
descriptions of \code{\link{calcBatchEffects}}  for
more detailed information about the data which should be contained in the
summary data.frame.
}
\examples{
## Shortly running example. For a more realistic example that takes
## some more time, run the same procedure with the full BEclearData
## dataset.

## Whole procedure that has to be done to use this function.
data(BEclearData)
ex.data <- ex.data[31:90, 7:26]
ex.samples <- ex.samples[7:26, ]

## Calculate the batch effects
batchEffects <- calcBatchEffects(data = ex.data, samples = ex.samples,
adjusted = TRUE, method = "fdr")
meds <- batchEffects$med
pvals <- batchEffects$pval

## Summarize p-values and median differences for batch affected genes
sum <- calcSummary(medians = meds, pvalues = pvals)

## Set values for summarized BEgenes to NA
clearedMatrix <- clearBEgenes(data = ex.data, samples = ex.samples, summary = sum)
}
\seealso{
\code{\link{calcBatchEffects}}

\code{\link{calcSummary}}

\code{\link{correctBatchEffect}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/correctBatchEffect.R
\name{correctBatchEffect}
\alias{correctBatchEffect}
\title{Correct a batch effect in DNA methylation data}
\usage{
correctBatchEffect(data, samples, adjusted=TRUE, method="fdr",
mediansTreshold = 0.05, pvaluesTreshold = 0.01, rowBlockSize=60, 
colBlockSize=60, epochs=50, lambda = 1, gamma = 0.01, r = 10,
outputFormat="", dir=getwd(), BPPARAM=SerialParam())
}
\arguments{
\item{data}{any matrix filled with beta values, column names have to be
sample_ids corresponding to the ids listed in "samples", row names have to
be gene names.}

\item{samples}{data frame with two columns, the first column has to contain
the sample numbers, the second column has to contain the corresponding batch
number. Colnames have to be named as "sample_id" and "batch_id".}

\item{adjusted}{should the p-values be adjusted or not, see "method" for
available adjustment methods.}

\item{method}{adjustment method for p-value adjustment, default
method is "false discovery rate adjustment", for other available methods see
the description of the used standard R package \code{\link{p.adjust}}. See
\code{\link{calcBatchEffects}} for more information.}

\item{mediansTreshold}{the threshold above or equal median values are regarded 
as batch effected, when the criteria for p-values is also met.}

\item{pvaluesTreshold}{the threshold below or equal p-values are regarded as 
batch effected, when the criteria for medians is also met.}

\item{rowBlockSize}{the number of rows that is used in a block if the
function is run in parallel mode and/or not on the whole matrix. Set this,
and the "colBlockSize" parameter to 0 if you want to run the function on the
whole input matrix. See \code{\link{imputeMissingData}} and especially the
details section of See \code{\link{imputeMissingData}} for more information
about this feature.}

\item{colBlockSize}{the number of columns that is used in a block if the
function is run in parallel mode and/or not on the whole matrix. Set this,
and the "rowBlockSize" parameter to 0 if you want to run the function on the
whole input matrix. See \code{\link{imputeMissingData}} and especially the
details section of See \code{\link{imputeMissingData}} for more information
about this feature.}

\item{epochs}{the number of iterations used in the gradient descent algorithm
to predict the missing entries in the data matrix. See
\code{\link{imputeMissingData}} for more information.}

\item{lambda}{constant that controls the extent of regularization during the
gradient descent}

\item{gamma}{constant that controls the extent of the shift of parameters
during the gradient descent}

\item{r}{length of the second dimension of variable matrices R and L}

\item{outputFormat}{you can choose if the finally returned data matrix should
be saved as an .RData file or as a tab-delimited .txt file in the specified
directory. Allowed values are "RData" and "txt".
See \code{\link{imputeMissingData}}
for more information.}

\item{dir}{set the path to a directory the predicted matrix should be stored.
The current working directory is defined as default parameter.}

\item{BPPARAM}{An instance of the
\code{\link[BiocParallel]{BiocParallelParam-class}} that determines how to
parallelisation of the functions will be evaluated.}
}
\value{
A list containing the following fields (for detailed information look
at the documentations of the corresponding functions):
\describe{
\item{medians}{A data.frame containing all median comparison values
corresponding to the input matrix.}
\item{pvalues}{A data.frame containing all p-values corresponding to the
input matrix.}
\item{summary}{The summarized results of the median comparison and p-value
calculation.}
\item{score.table}{A data.frame containing the number of found genes and a
BEscore for every batch.}
\item{cleared.data}{the input matrix with all values defined in the summary
set to NA.}
\item{predicted.data}{the input matrix after all previously NA values have
been predicted.}
\item{corrected.predicted.data}{the predicted matrix after the correction for
 predicted values outside the boundaries.}
}
}
\description{
This method combines most functions of the
\code{\link{BEclear-package}} to one. The method performs the whole process
of searching for batch effects and automatically correct them for a matrix
of beta values stemming from DNA methylation data.
}
\details{
correctBatchEffect

The function performs the whole process of searching for batch
effects and automatically correct them for a matrix of beta values stemming
from DNA methylation data. Thereby, the function is running most of the
functions from the \code{\link{BEclear-package}} in a logical order.\cr
First, median comparison values are calculated by the
\code{\link{calcBatchEffects}} function, followed by the calculation of p-values
also by the \code{\link{calcBatchEffects}} function. With the results from the median
comparison and p-value calculation, a summary data frame is build using the
\code{\link{calcSummary}} function, and a scoring table is established by
the \code{\link{calcScore}} function. Now, found entries from the summary are
set to NA in the input matrix using the \code{\link{clearBEgenes}} function,
then the \code{\link{imputeMissingData}} function is used to predict the
missing values and at the end, predicted entries outside the
boundaries (values lower than 0 or greater than 1) are corrected using the
\code{\link{replaceOutsideValues}} function.
}
\examples{
## Shortly running example. For a more realistic example that takes
## some more time, run the same procedure with the full BEclearData
## dataset.

## Whole procedure that has to be done to use this function.
## Correct the example data for a batch effect
data(BEclearData)
ex.data <- ex.data[31:90, 7:26]
ex.samples <- ex.samples[7:26, ]

# Note that row- and block sizes are just set to 10 to get a short runtime.
# To use these parameters, either use the default values or please note the
# description in the details section of \code{\link{imputeMissingData}}
result <- correctBatchEffect(
  data = ex.data, samples = ex.samples,
  adjusted = TRUE, method = "fdr", rowBlockSize = 10, colBlockSize = 10,
  epochs = 50, outputFormat = "RData", dir = getwd()
)

# Unlist variables
medians <- result$medians
pvals <- result$pvals
summary <- result$summary
score <- result$score.table
cleared <- result$clearedData
predicted <- result$predictedData
corrected <- result$correctedPredictedData
}
\references{
\insertRef{Akulenko2016}{BEclear}

\insertRef{Koren2009}{BEclear}

\insertRef{Candes2009}{BEclear}
}
\seealso{
\code{\link{calcBatchEffects}}

\code{\link{calcSummary}}

\code{\link{calcScore}}

\code{\link{clearBEgenes}}

\code{\link{imputeMissingData}}

\code{\link{replaceOutsideValues}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/imputeMissingDataForBlock.R
\name{imputeMissingDataForBlock}
\alias{imputeMissingDataForBlock}
\title{imputeMissingDataForBlock}
\usage{
imputeMissingDataForBlock(block, dir, epochs, lambda = 1, gamma = 0.01, r = 10)
}
\arguments{
\item{fixedSeed}{determines if they seed should be fixed, which is important
for testing}
}
\value{
number of the block processed
}
\description{
imputeMissingDataForBlock
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gdepoch.R
\name{gdepoch}
\alias{gdepoch}
\title{gdepoch}
\usage{
gdepoch(L, R, lambda, gamma, is, js, D, error_matrix = NULL)
}
\value{
a list containing two matrices generated by the gradient descent for
the current epoch
}
\description{
Run a gradient descent epoch (L and R are starting points,
gamma is stgammaize)
}
\keyword{internal}
